from django import template

# from openlink.core.lib import tools
from openlink.core.models import Mapping

register = template.Library()


@register.filter(name="url_link")
def url_link(obj):
    mapping = Mapping.objects.filter(foreign_id_obj=obj)
    return mapping



import json
import logging
import os.path
import shutil
import zipfile

import django_rq

# from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.urls import reverse
from guardian.decorators import permission_required_or_403
from openlink.core.connector import Mapper
from openlink.core.models import (
    Assay,
    Dataset,
    Investigation,
    Mappableobject,
    Mapping,
    MappingProjectUser,
    Project,
    Study,
    Tool,
)

logger = logging.getLogger(__name__)


@permission_required_or_403("change_project", (Project, "id", "project_id"))
def choose_object_to_publish(request, *args, **kwargs):
    # select objects to publish and get information from connector
    current_proj = kwargs["project_id"]
    project = get_object_or_404(Project, id=current_proj)
    investigation = get_object_or_404(Investigation, pk=kwargs["pk"])
    mappinginv = Mapping.objects.filter(foreign_id_obj__id=investigation.id)

    tags = []
    users = []
    listi = []
    mixedlist = []

    if request.method == "POST":
        if mappinginv:
            for name in mappinginv:
                tool_qs = get_object_or_404(Tool, mapping=name)
                connector2 = tool_qs.get_connector()
                if issubclass(connector2.__class__, Mapper):
                    information = connector2.get_information(name.type, name.object_id)
                    if "tags" in information:
                        for tag in information["tags"]:
                            tags.append(tag)
                    if "user" in information:
                        for user in information["user"]:
                            users.append(user)

        tomap = []
        for item in request.POST:
            if item != "csrfmiddlewaretoken":
                tomap.append(item)
                mappi = Mapping.objects.filter(foreign_id_obj__id=item, type="dataset")
                if mappi:
                    name = mappi[0]
                    mixedlist.append(name)
                    tool_qs = get_object_or_404(Tool, mapping=name)
                    connector2 = tool_qs.get_connector()
                    if issubclass(connector2.__class__, Mapper):
                        information = connector2.get_information(
                            name.type, name.object_id
                        )
                        if "tags" in information:
                            for tag in information["tags"]:
                                tags.append(tag)
                        if "user" in information:
                            for user in information["user"]:
                                users.append(user)
                try:
                    name = Study.objects.get(pk=item)
                except Study.DoesNotExist:
                    try:
                        name = Assay.objects.get(pk=item)
                    except Assay.DoesNotExist:
                        name = Dataset.objects.get(pk=item)
                listi.append(name)
                mixedlist.append(name)
                if mappi:
                    mixedlist.pop()
        tagstring = ""
        for tag in tags:
            tagstring += tag
            tagstring += ","
        openlinkusers = MappingProjectUser.objects.filter(project=project)
        for openlinkuser in openlinkusers:
            users.append(str(openlinkuser.user))
        users = list(set(users))
        # fmt: off
        for iduser, user in enumerate(users[:-1]):
            dec = 0
            try:
                users[iduser]
            except Exception:
                break
            forcomp = []
            if " " in users[iduser]:
                usersep = users[iduser].split(" ")
                forcomp.append((usersep[1] + " " + usersep[0]).lower())  # 2
                forcomp.append((usersep[1][0] + " " + usersep[0]).lower())  # 3
                forcomp.append((usersep[0][0] + " " + usersep[1]).lower())  # 4
                forcomp.append((usersep[1] + " " + usersep[0][0]).lower())  # 5
                forcomp.append((usersep[0] + " " + usersep[1][0]).lower())  # 6
                forcomp.append((usersep[1][0] + usersep[0]).lower())  # 7
                forcomp.append((usersep[0][0] + usersep[1]).lower())  # 8
                forcomp.append((usersep[1] + usersep[0][0]).lower())  # 9
                forcomp.append((usersep[0] + usersep[1][0]).lower())  # 10
            forcomp.append(users[iduser].lower())  # 1
            for idcomp, compa in enumerate(users[iduser + 1:]):
                for comp in forcomp:
                    if comp == users[idcomp + iduser + 1 - dec].lower():
                        if len(users[idcomp + iduser + 1 - dec]) > len(users[iduser]):
                            users[iduser] = users[idcomp + iduser + 1 - dec]
                        users.pop(idcomp + iduser + 1 - dec)
                        dec += 1
        # fmt: on
        logging.debug(mixedlist)
        return render(
            request,
            "publish/choose_metadata.html",
            {
                "title": investigation,
                "tags": tagstring,
                "list": listi,
                "listmap": mixedlist,
                "users": users,
                "tomap": tomap,
                "project": project,
            },
        )
    return render(
        request,
        "publish/choose_object_to_publish.html",
        {"items": investigation, "project": project},
    )


def zipdir(path, ziph):
    # ziph is zipfile handle
    for root, dirs, files in os.walk(path):
        for file in files:
            ziph.write(
                os.path.join(root, file),
                os.path.relpath(os.path.join(root, file), os.path.join(path, "..")),
            )


@permission_required_or_403("change_project", (Project, "id", "project_id"))
def choose_metadata(request, *args, **kwargs):
    # get metadata and start asynchronous publication
    current_proj = kwargs["project_id"]
    project = get_object_or_404(Project, id=current_proj)
    if request.method == "POST":
        queue = django_rq.get_queue("default")
        queue.enqueue(tool_to_tool, request.POST, project, kwargs)
        return HttpResponseRedirect(
            reverse("core:projects-details", kwargs={"project_id": current_proj})
        )


def tool_to_tool(request, project, kwargs):
    # download dataset and create depo with metadata
    tabstring = request["datatoup"]
    tabstring = tabstring.replace("'", '"')
    tab = json.loads(tabstring)
    title = request["title"]
    path = "openlink/core/lib/tools/tmp"
    if not os.path.exists("%s/%s" % (path, title)):
        os.makedirs("%s/%s" % (path, title))
        path = "%s/%s" % (path, title)
    level = ("investigation", "study", "assay")
    intlevel = 0
    for item in tab:
        mappi = Mapping.objects.filter(foreign_id_obj__id=item, type="dataset")
        if mappi.exists():
            name = mappi[0]
            tool_qs = get_object_or_404(Tool, mapping=name)
            connector2 = tool_qs.get_connector()
            object_id = name.object_id
        try:
            name = Study.objects.get(pk=item)
            level_ob = "study"
        except Study.DoesNotExist:
            try:
                name = Assay.objects.get(pk=item)
                level_ob = "assay"
            except Assay.DoesNotExist:
                name = Dataset.objects.get(pk=item)
                level_ob = "dataset"
        if level_ob == "dataset":
            # Download datasets
            if mappi.exists():
                try:
                    connector2.download(object_id, path)
                except Exception:
                    pass

        elif level[intlevel] == level_ob:
            path = path.rsplit("/", 1)[0]
            if not os.path.exists("%s/%s" % (path, name)):
                os.makedirs("%s/%s" % (path, name))
                path = "%s/%s" % (path, name)
        try:
            if level[intlevel - 1] == level_ob:
                path = path.rsplit("/", 1)[0]
                path = path.rsplit("/", 1)[0]
                if not os.path.exists("%s/%s" % (path, name)):
                    os.makedirs("%s/%s" % (path, name))
                    path = "%s/%s" % (path, name)
                intlevel -= 1
        except Exception:
            pass
        try:
            if level[intlevel + 1] == level_ob:
                if not os.path.exists("%s/%s" % (path, name)):
                    os.makedirs("%s/%s" % (path, name))
                    path = "%s/%s" % (path, name)
                    intlevel += 1
        except Exception:
            pass
    mappi = None
    # Create zipfile
    zipf = zipfile.ZipFile(
        "openlink/core/lib/tools/tmp/%s.zip" % title,
        "w",
        zipfile.ZIP_DEFLATED,
    )
    zipdir("openlink/core/lib/tools/tmp/%s" % title, zipf)
    zipf.close()
    shutil.rmtree("openlink/core/lib/tools/tmp/%s" % title)
    tagtab = request["tags"].split(",")
    tools_qs = Tool.objects.filter(project=project, id=kwargs["tool_id"])
    for tool in tools_qs:
        connector = tool.get_connector()
    creator = {}
    creators = []
    for post in request:
        if post.startswith("auth"):
            creator = {"name": request[post]}
        if post.startswith("affi"):
            if request[post]:
                creator.update({"affiliation": request[post]})
            creators.append(creator)
    metadata = {
        "metadata": {
            "title": title,
            "upload_type": request["type"],
            "description": request["description"],
            "creators": creators,
            "keywords": tagtab,
        }
    }
    # Add metadata to deposit
    if "saved_doi" not in kwargs:
        r = connector.create_empty_depo()
        path_to_file = "openlink/core/lib/tools/tmp/%s.%s" % (title, "zip")
        connector.add_file_to_depo(r, path_to_file)
        jsonr = connector.add_metadata_to_depo(metadata, r["links"]["self"])

        r = connector.publish_depo(jsonr)

        listitem = (
            request["list"]
            .replace("[", "")
            .replace("]", "")
            .replace("'", "")
            .split(",")
        )
        for item in listitem:
            try:
                mappi = Mapping.objects.filter(foreign_id_obj__id=item, type="dataset")
                if mappi.exists():
                    origin_object = get_object_or_404(Mappableobject, id=item)
                    new_mapping_object = Mapping()
                    new_mapping_object.name = title
                    new_mapping_object.tool_id = tool
                    new_mapping_object.type = "dataset"
                    new_mapping_object.object_id = r.json()["doi"]
                    new_mapping_object.foreign_id_obj = origin_object
                    new_mapping_object.save()
            except Dataset.DoesNotExist:
                pass

        new_mapping_inv = Mapping()
        new_mapping_inv.name = title
        new_mapping_inv.tool_id = tool
        new_mapping_inv.type = "investigation"
        new_mapping_inv.object_id = r.json()["doi"]
        new_mapping_inv.foreign_id_obj = get_object_or_404(
            Mappableobject, id=kwargs["pk"]
        )
        new_mapping_inv.save()

        os.remove("openlink/core/lib/tools/tmp/%s.%s" % (title, "zip"))
    else:
        r = connector.update_depo(str(kwargs["saved_doi"]), metadata)
    return HttpResponseRedirect(
        reverse(
            "core:projects-details",
            kwargs={"project_id": kwargs["project_id"]},
        )
    )



import logging

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit
from django.apps import apps
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseForbidden, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.urls import reverse
from django.utils import timezone
from django.views.decorators.http import require_GET, require_POST
from django.views.generic.edit import UpdateView
from guardian.decorators import permission_required_or_403
from openlink.core import connector as tools
from openlink.core.connector import Mapper, Publisher
from openlink.core.forms import LinkToolForm, SelectToolForm, ToolForm, ToolProjectForm
from openlink.core.models import Mapping, Profile, Project, Tool, Toolparam

logger = logging.getLogger(__name__)


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def list_tools_project(request, project_id):
    # get list of tools linked to a project
    project = get_object_or_404(Project, id=project_id)
    list_tools_project = Tool.objects.filter(project__id=project_id)
    return render(
        request,
        "tools/list_tool_project.html",
        {"list_tools_project": list_tools_project, "project": project},
    )


@login_required
@require_GET
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def select_connector_project(request, project_id):
    # Get selected connector and return the suited tool form
    project = get_object_or_404(Project, id=project_id)
    if "connector" not in request.GET:
        form = ToolForm()
        return render(
            request,
            "tools/new-choose-connector-project.html",
            {"form": form, "project": project},
        )
    else:
        form = ToolForm(request.GET)
        form.project_id = project_id
        if form.is_valid():
            connector_class = tools.get_connector_class(request.GET["connector"])
            tool_name = request.GET["name"]
            if connector_class is None:
                # TODO: add a message (https://docs.djangoproject.com/fr/3.0/ref/contrib/messages/#using-messages-in-views-and-templates)
                return HttpResponseRedirect(
                    reverse("core:add-tool-project", args=[project_id])
                )
            return render(
                request,
                "tools/add_tool_project.html",
                {
                    "form": connector_class.get_creation_form(),
                    "connector": connector_class.__name__,
                    "tool_name": tool_name,
                    "project": project,
                    "logo": connector_class.get_logo,
                },
            )
        else:
            messages.error(
                request, "unable to select " + request.GET["name"] + " tool connector"
            )
            return render(
                request,
                "tools/new-choose-connector-project.html",
                {"form": form, "project": project},
            )


@login_required
@require_POST
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def add_tool_project(request, connector, project_id):
    # Display tool creation form  and create tool if the form is valid
    connector_class = tools.get_connector_class(connector)
    form = connector_class.get_creation_form(request.POST)
    project = get_object_or_404(Project, id=project_id)
    form.project_id = project_id
    try:
        if not form.is_valid():
            logger.debug(request.POST.get("name"))
            messages.error(
                request, "unable to add " + connector_class.get_name() + " tool"
            )
            return render(
                request,
                "tools/add_tool_project.html",
                {
                    "form": form,
                    "connector": connector,
                    "project": project,
                    "logo": connector_class.get_logo,
                    "tool_name": request.POST.get("name"),
                },
            )
    except Exception as e:
        messages.error(request, str(e))
        return render(
            request,
            "tools/add_tool_project.html",
            {
                "form": form,
                "connector": connector,
                "project": project,
                "logo": connector_class.get_logo,
                "tool_name": request.POST.get("name"),
            },
        )

    if form.is_valid():
        profile = Profile.objects.get(user=request.user)

        # Save new Tool using form entries
        logger.debug(request.POST)
        tool = Tool(
            name=request.POST.get("name"),
            connector=connector_class.__name__,
            author=profile,
            date_created=timezone.now(),
            project=project,
        )
        tool.save()
        for param in form.cleaned_data:
            url_param = Toolparam(
                tool_id=tool, key=param, value=form.cleaned_data[param]
            )
            url_param.save()
    # TODO: add a message (https://docs.djangoproject.com/fr/3.0/ref/contrib/messages/#using-messages-in-views-and-templates)
    return HttpResponseRedirect(reverse("core:tools-project", args=[project_id]))


@login_required
# @require_POST
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def toolsparam_edit_project(request, *args, **kwargs):
    # Update tool parameters
    tool = get_object_or_404(Tool, pk=kwargs["tool_id"])
    project = get_object_or_404(Project, pk=kwargs["project_id"])
    connector = tool.connector
    connector_class = tools.get_connector_class(connector)
    if request.method == "POST":
        form = connector_class.get_edition_form(request.POST)
        toolsparam = get_object_or_404(Toolparam, tool_id=kwargs["tool_id"], key="url")
        form.set_url(toolsparam.value)
        if not form.is_valid():
            messages.error(
                request, "unable to add " + connector_class.get_name() + " tool"
            )
            return render(
                request,
                "tools/edit_toolparam_project.html",
                {
                    "form": form,
                    "connector": connector_class.get_name(),
                    "project": project,
                    "tool": tool,
                    "logo": connector_class.get_logo,
                },
            )
        if form.is_valid():
            for param in form.cleaned_data:
                instance_param = Toolparam.objects.get(tool_id=tool, key=param)
                instance_param.value = form.cleaned_data[param]
                instance_param.save()

        # TODO: add a message (https://docs.djangoproject.com/fr/3.0/ref/contrib/messages/#using-messages-in-views-and-templates)
        messages.info(request, "Tool parameters updated")
        return HttpResponseRedirect(reverse("core:tools-project", args=[project.id]))
    else:
        form = connector_class.get_edition_form()
        return render(
            request,
            "tools/edit_toolparam_project.html",
            {
                "form": form,
                "connector": connector_class.get_name(),
                "project": project,
                "tool": tool,
                "logo": connector_class.get_logo,
            },
        )


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def get_tool_info_project(request, *args, **kwargs):
    # get tool details for a given OpenLink poject
    project = get_object_or_404(Project, id=kwargs["project_id"])
    t_list = Tool.objects.filter(pk=kwargs["tool_id"])
    context = {"tool_list": t_list, "project": project}
    return render(request, "tools/tool_info_project.html", context)


class ToolUpdate(UpdateView):
    # Generic editing view for a given Experience
    model = Tool
    form_class = ToolProjectForm
    template_name = "tools/tool_update_form.html"
    pk_url_kwarg = "tool_id"

    def __init__(self, *args, **kwargs):
        super(ToolUpdate, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_group_wrapper_class = "row"
        self.helper.label_class = "col-sm-offset-1 col-sm-2"
        self.helper.field_class = "col-md-8"

    def get_context_data(self, **kwargs):
        project_id = self.kwargs["project_id"]
        context = super().get_context_data(**kwargs)
        context["name"] = self.object.name
        context["id"] = self.object.id
        context["tool"] = self.object
        project = get_object_or_404(Project, id=project_id)
        context["project"] = project
        return context

    def get_form(self, form_class=None):
        form = super().get_form(form_class)
        form.helper = FormHelper()
        form.helper.form_group_wrapper_class = "row"
        form.helper.label_class = "col-sm-offset-1 col-sm-2"
        form.helper.field_class = "col-md-8"
        form.helper.add_input(Submit("submit", "Update", css_class="btn-primary"))
        return form

    def dispatch(self, request, *args, **kwargs):
        return super(ToolUpdate, self).dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        project_id = self.kwargs["project_id"]
        tool_id = self.kwargs["tool_id"]
        project = get_object_or_404(Project, id=project_id)
        if not self.request.user.has_perm("change_project", project):
            return HttpResponseForbidden()
        if form.is_valid():
            tool = form.save(commit=False)
            tool.save()
            messages.info(self.request, "Tool " + str(tool.name) + " saved")
            return HttpResponseRedirect(
                reverse("core:tools-project", args=[project.id])
            )
        else:
            return HttpResponseRedirect(
                reverse("core:tools-edit-project", args=[project_id, tool_id])
            )


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def delete_tool(request, *args, **kwargs):
    # delete selected tool
    obj_id = kwargs["pk"]
    project_id = kwargs["project_id"]
    data_type = "tool"
    Model = apps.get_model("core", data_type.capitalize())
    if Model.objects.filter(author__user=request.user, id=obj_id).exists():
        obj_to_delete = Model.objects.filter(author__user=request.user).get(id=obj_id)

    if request.method == "POST":
        obj_to_delete.delete()
        return HttpResponseRedirect(
            reverse("core:tools-project", kwargs={"project_id": project_id})
        )

    else:
        if data_type == "tool":
            form = LinkToolForm
    return render(
        request,
        "mapping/form_delete.html",
        {"form": form, "obj": obj_to_delete, "data_type": data_type},
    )


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def choose_tool(request, *args, **kwargs):
    # select a tool for the mapping of an OpenLink object
    tool_list = []
    if "investigation_id" in kwargs:
        current_inv = kwargs["investigation_id"]
        data_type = kwargs["data_type"]
    else:
        data_type = "investigation"
    current_proj = kwargs["project_id"]
    project = get_object_or_404(Project, id=current_proj)
    exist_tool = 0
    project_tools = Tool.objects.filter(project=project)
    pk = kwargs["pk"]
    for p in project_tools:
        exist_tool = 1
        connector = p.get_connector()
        if issubclass(connector.__class__, Mapper):
            list_ressource = connector.get_supported_types()
            if data_type in list_ressource:
                if data_type == "investigation":
                    try:
                        mo_list = Mapping.objects.get(
                            foreign_id_obj__investigation__id=pk, tool_id=p
                        )
                    except Mapping.DoesNotExist:
                        mo_list = None
                if data_type == "study":
                    try:
                        mo_list = Mapping.objects.get(
                            foreign_id_obj__study__id=pk, tool_id=p
                        )
                    except Mapping.DoesNotExist:
                        mo_list = None
                elif data_type == "assay":
                    try:
                        mo_list = Mapping.objects.get(
                            foreign_id_obj__assay__id=pk, tool_id=p
                        )
                    except Mapping.DoesNotExist:
                        mo_list = None
                elif data_type == "dataset":
                    try:
                        mo_list = Mapping.objects.get(
                            foreign_id_obj__dataset__id=pk, tool_id=p
                        )
                    except Mapping.DoesNotExist:
                        mo_list = None
                if mo_list is None:
                    tool_list.append(p)

    if request.method == "POST":
        host = request.POST["object_name"]
        result = Tool.objects.filter(id=int(host))

        for dp in result:
            tool_id = int(dp.id)
            # tag_tool = dp.tag_tool

            # kwargs['tool'] = str(dp['tool'])
        if "investigation_id" in kwargs:
            return HttpResponseRedirect(
                reverse(
                    "core:choose_non_linked_object_tool",
                    kwargs={
                        "project_id": kwargs["project_id"],
                        "investigation_id": current_inv,
                        "data_type": data_type,
                        "pk": kwargs["pk"],
                        "tool_id": tool_id,
                    },
                )
            )
        else:
            return HttpResponseRedirect(
                reverse(
                    "core:choose_non_linked_object_tool",
                    kwargs={
                        "project_id": kwargs["project_id"],
                        "data_type": data_type,
                        "pk": kwargs["pk"],
                        "tool_id": tool_id,
                    },
                )
            )

    else:
        form = SelectToolForm(instance=tool_list)

    if "investigation_id" in kwargs:
        return render(
            request,
            "tools/form_choose_tool.html",
            {
                "form": form,
                "project": project,
                "exist_tool": exist_tool,
                "tool_list": tool_list,
                "current_inv": current_inv,
                "data_type": data_type,
                "pk": pk,
                "tool_list": tool_list,
            },
        )
    else:
        return render(
            request,
            "tools/form_choose_tool.html",
            {
                "form": form,
                "project": project,
                "exist_tool": exist_tool,
                "tool_list": tool_list,
                "data_type": data_type,
                "pk": pk,
                "tool_list": tool_list,
            },
        )


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def choose_tool_for_publish(request, *args, **kwargs):
    # Select a tool where to publish data
    tool_list = []
    data_type = "investigation"
    current_proj = kwargs["project_id"]
    project = get_object_or_404(Project, id=current_proj)
    project_tools = Tool.objects.filter(project=project)
    pk = kwargs["pk"]

    for p in project_tools:
        connector = p.get_connector()
        if issubclass(connector.__class__, Publisher):
            tool_list.append(p)

    if request.method == "POST":
        host = request.POST["object_name"]
        result = Tool.objects.filter(id=int(host))
        for dp in result:
            tool_id = int(dp.id)
            tag_tool = dp.tag_tool
        return HttpResponseRedirect(
            reverse(
                "core:choose_object_to_publish",
                kwargs={
                    "project_id": kwargs["project_id"],
                    "data_type": data_type,
                    "pk": kwargs["pk"],
                    "tool": tag_tool,
                    "tool_id": tool_id,
                },
            )
        )

    else:
        form = SelectToolForm(instance=tool_list)
        return render(
            request,
            "tools/form_choose_tool.html",
            {
                "form": form,
                "project": project,
                "data_type": data_type,
                "pk": pk,
                "tool_list": tool_list,
                "function": "publish",
            },
        )



from django import template

# from django.conf import settings
from openlink import __version__

register = template.Library()


@register.filter(name="get_version")
def get_version(version):
    return version + __version__



from django import template
from openlink.core.models import Tool

# from openlink.core.lib.tools import
register = template.Library()


@register.filter(name="has_access_url")
def has_access_url(instance):
    connector = Tool.objects.get(id=instance.id).get_connector()
    return connector.has_access_url()



from django.contrib.auth.decorators import login_required
from django.shortcuts import render
from openlink.core.models import Profile, Tool


def home(request):
    return render(request, "openlink/home.html")


def contact(request):
    return render(request, "openlink/contact.html")


@login_required
def get_user_profile(request, username):
    p_list = Profile.objects.filter(user=request.user)
    t_list = Tool.objects.filter(author__user=request.user)
    context = {"username": username, "profile_list": p_list, "tool_list": t_list}
    return render(request, "openlink/user_profile.html", context)



from django import template
from openlink.core.views.projects import humansize

register = template.Library()


@register.filter(name="humansize")
def humansizes(nbytes):
    if isinstance(nbytes, int):
        return humansize(nbytes)
    return nbytes



from openlink.core.models import (
    Assay,
    Dataset,
    Investigation,
    Mappableobject,
    Mapping,
    Study,
)
from rest_framework import serializers


class MappingSerializer(serializers.ModelSerializer):
    class Meta:
        model = Mapping
        fields = ["name", "tool_id", "object_id", "foreign_id_obj"]
        depth = 2


class MappableobjectSerializer(serializers.ModelSerializer):
    class Meta:
        model = Mappableobject
        fields = ["id"]
        depth = 1


class DatasetSerialiser(MappableobjectSerializer):
    data = serializers.SerializerMethodField()

    class Meta(MappableobjectSerializer.Meta):
        model = Dataset
        fields = ["data"]

    def get_data(self, obj):
        list_map = []

        qs_maps = Mapping.objects.filter(foreign_id_obj=obj.id)
        for qs_map in qs_maps:
            obj_type = qs_map.type
            obj_id = qs_map.object_id
            connector = qs_map.tool_id.get_connector()
            txt_to_add = {
                "tool": qs_map.tool_id.name,
                "tool_id": qs_map.tool_id.id,
                "obj_id": obj_id,
                "url": connector.get_url_link_to_an_object(obj_type, obj_id),
            }
            list_map.append(txt_to_add)

        return {
            "type": obj_type,
            "id": obj.pk,
            "name": obj.name,
            "mapping": list_map,
        }


class AssaySerializer(MappableobjectSerializer):
    dataset = DatasetSerialiser(many=True)
    data = serializers.SerializerMethodField()

    class Meta(MappableobjectSerializer.Meta):
        model = Assay
        fields = ["data", "dataset"]

    def get_data(self, obj):
        list_map = []

        qs_maps = Mapping.objects.filter(foreign_id_obj=obj.id)
        for qs_map in qs_maps:
            obj_type = qs_map.type
            obj_id = qs_map.object_id
            connector = qs_map.tool_id.get_connector()
            txt_to_add = {
                "tool": qs_map.tool_id.name,
                "tool_id": qs_map.tool_id.id,
                "obj_id": obj_id,
                "url": connector.get_url_link_to_an_object(obj_type, obj_id),
            }
            list_map.append(txt_to_add)

        return {
            "type": obj_type,
            "id": obj.pk,
            "name": obj.name,
            "mapping": list_map,
        }


class StudySerializer(MappableobjectSerializer):
    assay = AssaySerializer(many=True)
    data = serializers.SerializerMethodField()

    class Meta(MappableobjectSerializer.Meta):
        model = Study
        fields = ["data", "assay"]

    def get_data(self, obj):
        list_map = []

        qs_maps = Mapping.objects.filter(foreign_id_obj=obj.id)
        for qs_map in qs_maps:
            obj_type = qs_map.type
            obj_id = qs_map.object_id
            connector = qs_map.tool_id.get_connector()
            txt_to_add = {
                "tool": qs_map.tool_id.name,
                "tool_id": qs_map.tool_id.id,
                "obj_id": obj_id,
                "url": connector.get_url_link_to_an_object(obj_type, obj_id),
            }
            list_map.append(txt_to_add)

        return {
            "type": obj_type,
            "id": obj.pk,
            "name": obj.name,
            "mapping": list_map,
        }


class InvestigationSerializer(MappableobjectSerializer):
    study = StudySerializer(many=True)
    dataset = DatasetSerialiser(many=True)
    data = serializers.SerializerMethodField()

    class Meta(MappableobjectSerializer.Meta):
        model = Investigation
        fields = ["data", "study", "dataset"]

    def get_data(self, obj):
        list_map = []
        qs_maps = Mapping.objects.filter(foreign_id_obj=obj.id)
        for qs_map in qs_maps:
            obj_type = qs_map.type
            obj_id = qs_map.object_id
            connector = qs_map.tool_id.get_connector()
            txt_to_add = {
                "tool": qs_map.tool_id.name,
                "tool_id": qs_map.tool_id.id,
                "obj_id": obj_id,
                "url": connector.get_url_link_to_an_object(obj_type, obj_id),
            }
            list_map.append(txt_to_add)

        return {
            "type": "investigation",
            "id": obj.pk,
            "name": obj.name,
            "mapping": list_map,
        }


class ProjectSerializer(MappableobjectSerializer):
    investigation = InvestigationSerializer(many=True)
    data = serializers.SerializerMethodField()

    class Meta(MappableobjectSerializer.Meta):
        model = Investigation
        fields = ["data", "investigation"]

    def get_data(self, obj):
        list_map = []
        qs_maps = Mapping.objects.filter(foreign_id_obj=obj.id)
        for qs_map in qs_maps:
            obj_type = qs_map.type
            obj_id = qs_map.object_id
            connector = qs_map.tool_id.get_connector()
            txt_to_add = {
                "tool": qs_map.tool_id.name,
                "tool_id": qs_map.tool_id.id,
                "obj_id": obj_id,
                "url": connector.get_url_link_to_an_object(obj_type, obj_id),
            }
            list_map.append(txt_to_add)

        return {
            "type": "project",
            "id": obj.pk,
            "name": obj.name,
            "mapping": list_map,
        }



import logging
from datetime import datetime

from ckeditor.fields import RichTextField
from django.contrib.auth.models import User
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.forms import ModelChoiceField
from django.shortcuts import get_object_or_404
from django.utils import timezone
from openlink.core.connector import get_connector

logger = logging.getLogger(__name__)


class Team(models.Model):
    name = models.CharField(max_length=100, blank=True)
    date = models.DateTimeField(default=datetime.now, blank=True, null=True)

    def __str__(self):
        return self.name


class Profile(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    team = models.ManyToManyField(Team, blank=True)

    def __str__(self):
        return str(self.user)


@receiver(post_save, sender=User)
def create_user_profile(sender, instance, created, **kwargs):
    if created:
        Profile.objects.create(user=instance)


@receiver(post_save, sender=User)
def save_user_profile(sender, instance, **kwargs):
    instance.profile.save()


class Mappableobject(models.Model):
    def __str__(self):
        return str(self.id)

    @property
    def type(self):
        return self._meta.model_name

    @classmethod
    def create(cls, form, profile, parent_id):
        logger.debug("create")
        item = form.save(commit=False)
        item.author = profile
        item.date_created = timezone.now()
        item.save()

        parent_type = cls.get_parent_type()
        parent_item = getattr(parent_type, "get_" + parent_type.__name__.lower())(
            parent_id
        ).first()

        getattr(parent_item, item.get_plural()).add(item)
        parent_item.save()

        return item


class Image(Mappableobject):
    name = models.CharField(max_length=100, blank=True)
    author = models.ForeignKey(Profile, on_delete=models.CASCADE, null=True)

    def __str__(self):
        return self.name


class Dataset(Mappableobject):
    name = models.CharField(max_length=100)
    description = RichTextField(
        blank=True, null=True, config_name="zenodo"
    )  # models.TextField(blank=True, null=True)
    author = models.ForeignKey(Profile, on_delete=models.CASCADE, null=True)
    date_created = models.DateTimeField(blank=True, null=True)
    date_modified = models.DateTimeField(blank=True, null=True)

    def __str__(self):
        return self.name

    def get_plural(self):
        return "datasets"

    @staticmethod
    def get_parent(id):
        return Assay.objects.filter(datasets__id=id)

    def get_dataset(id):
        return Dataset.objects.filter(id=id)

    def get_assay(id):
        id = id.id if type(id) == Dataset else id
        return Assay.objects.filter(datasets__id=id)

    def get_study(id):
        id = id.id if type(id) == Dataset else id
        return Study.objects.filter(assays__datasets__id=id)

    def get_investigation(id):
        id = id.id if type(id) == Dataset else id
        return Investigation.objects.filter(studies__assays__datasets__id=id)

    def get_project(id):
        id = id.id if type(id) == Dataset else id
        return Project.objects.filter(investigations__studies__assays__datasets__id=id)

    def get_parent_type():
        return Assay

    def get_child_type():
        return None

    def model_plural():
        return "datasets"

    @classmethod
    def create_instance(cls, child_data_object, author):
        New_dataset = cls()
        New_dataset.name = child_data_object.name
        New_dataset.description = child_data_object.description
        New_dataset.author = author
        New_dataset.date_created = timezone.now()
        return New_dataset


class Assay(Mappableobject):
    name = models.CharField(max_length=100)
    description = RichTextField(
        blank=True, null=True, config_name="zenodo"
    )  # models.TextField(blank=True, null=True)
    author = models.ForeignKey(Profile, on_delete=models.CASCADE, null=True)
    date_created = models.DateTimeField(blank=True, null=True)
    date_modified = models.DateTimeField(blank=True, null=True)
    datasets = models.ManyToManyField(Dataset, blank=True)

    def __str__(self):
        return self.name

    def get_plural(self):
        return "assays"

    @staticmethod
    def get_parent(id):
        return Study.objects.filter(assays__id=id)

    def get_dataset(id):
        return Dataset.objects.filter(assay__id=id)

    def get_assay(id):
        return Assay.objects.filter(id=id)

    def get_study(id):
        id = id.id if type(id) == Assay else id
        return Study.objects.filter(assays__id=id)

    def get_investigation(id):
        id = id.id if type(id) == Assay else id
        return Investigation.objects.filter(studies__assays__id=id)

    def get_project(id):
        id = id.id if type(id) == Assay else id
        return Project.objects.filter(investigations__studies__assays__id=id)

    @property
    def items(self):
        return list(self.datasets.all())

    def get_parent_type():
        return Study

    def get_child_type():
        return Dataset

    def model_plural():
        return "assays"

    @classmethod
    def create_instance(cls, child_data_object, author):
        New_assay = cls()
        New_assay.name = child_data_object.name
        New_assay.description = child_data_object.description
        New_assay.author = author
        New_assay.date_created = timezone.now()
        return New_assay

    def link_child_to_parent(parent_data_object, child_data_object):
        parent_data_object.datasets.add(child_data_object)


class Study(Mappableobject):
    name = models.CharField(max_length=100)
    description = RichTextField(
        blank=True, null=True, config_name="zenodo"
    )  # models.TextField(blank=True, null=True)
    author = models.ForeignKey(
        Profile, on_delete=models.CASCADE, null=True, related_name="studies"
    )
    date_created = models.DateTimeField(blank=True, null=True)
    date_modified = models.DateTimeField(blank=True, null=True)
    assays = models.ManyToManyField(Assay, blank=True)

    def __str__(self):
        return self.name

    @property
    def items(self):
        return list(self.assays.all())

    def get_plural(self):
        return "studies"

    @staticmethod
    def get_parent(id):
        return Investigation.objects.filter(studies__id=id)

    def get_dataset(id):
        return Dataset.objects.filter(assay__study__id=id)

    def get_assay(id):
        return Assay.objects.filter(study__id=id)

    def get_study(id):
        return Study.objects.filter(id=id)

    def get_investigation(id):
        id = id.id if type(id) == Study else id
        return Investigation.objects.filter(studies__id=id)

    def get_project(id):
        id = id.id if type(id) == Study else id
        return Project.objects.filter(investigations__studies__id=id)

    def get_parent_type():
        return Investigation

    def get_child_type():
        return Assay

    def model_plural():
        return "studies"

    @classmethod
    def create_instance(cls, child_data_object, author):
        New_study = cls()
        New_study.name = child_data_object.name
        New_study.description = child_data_object.description
        New_study.author = author
        New_study.date_created = timezone.now()
        return New_study

    def link_child_to_parent(parent_data_object, child_data_object):
        parent_data_object.assays.add(child_data_object)


class Investigation(Mappableobject):
    name = models.CharField(max_length=100)
    description = RichTextField(
        blank=True, null=True, config_name="zenodo"
    )  # models.TextField(blank=True, null=True)
    author = models.ForeignKey(Profile, on_delete=models.CASCADE, null=True)
    date_created = models.DateTimeField(blank=True, null=True)
    date_modified = models.DateTimeField(blank=True, null=True)
    studies = models.ManyToManyField(Study, blank=True)

    def __str__(self):
        return self.name

    @property
    def items(self):
        return list(self.studies.all())

    def get_plural(self):
        return "investigations"

    @staticmethod
    def get_parent(id):
        return Project.objects.filter(investigations__id=id)

    def get_dataset(id):
        return Dataset.objects.filter(assay__study__investigation__id=id)

    def get_assay(id):
        return Assay.objects.filter(study__investigation__id=id)

    def get_study(id):
        return Study.objects.filter(investigation__id=id)

    def get_investigation(id):
        return Investigation.objects.filter(id=id)

    def get_project(id):
        id = id.id if type(id) == Investigation else id
        return Project.objects.filter(investigations__id=id)

    def get_parent_type():
        return Project

    def get_child_type():
        return Study

    def model_plural():
        return "investigations"

    @classmethod
    def create_instance(cls, child_data_object, author):
        New_investigation = cls()
        New_investigation.name = child_data_object.name
        New_investigation.description = child_data_object.description
        New_investigation.author = author
        New_investigation.date_created = timezone.now()
        return New_investigation

    def link_child_to_parent(parent_data_object, child_data_object):
        parent_data_object.studies.add(child_data_object)


class Project(Mappableobject):
    name = models.CharField(max_length=100)
    description = RichTextField(
        blank=True, null=True, config_name="zenodo"
    )  # models.TextField(blank=True, null=True)
    author = models.ForeignKey(Profile, on_delete=models.CASCADE, null=True)
    date_created = models.DateTimeField(blank=True, null=True)
    date_modified = models.DateTimeField(blank=True, null=True)
    investigations = models.ManyToManyField(Investigation, blank=True)

    class Meta:
        permissions = (("manage_user", "Manage user"),)

    def __str__(self):
        return self.name

    def get_plural(self):
        return "projects"

    def get_dataset(id):
        return Dataset.objects.filter(assay__study__investigation__project__id=id)

    def get_assay(id):
        return Assay.objects.filter(study__investigation__project__id=id)

    def get_study(id):
        return Study.objects.filter(investigation__project__id=id)

    def get_investigation(id):
        return Investigation.objects.filter(project__id=id)

    def get_project(id):
        return Project.objects.filter(id=id)

    @property
    def items(self):
        return list(self.investigations.all())

    def get_child_type():
        return Investigation

    def model_plural():
        return "projects"

    @classmethod
    def create_instance(cls, child_data_object, author):
        New_project = cls()
        New_project.name = child_data_object.name
        New_project.description = child_data_object.description
        New_project.author = author
        New_project.date_created = timezone.now()
        return New_project

    def link_child_to_parent(parent_data_object, child_data_object):
        parent_data_object.investigations.add(child_data_object)


class MappingProjectUser(models.Model):
    ADMINISTRATOR = "administrator"
    CONTRIBUTOR = "contributor"
    COLLABORATOR = "collaborator"
    STATUS = [
        (ADMINISTRATOR, "administrator"),
        (CONTRIBUTOR, "contributor"),
        (COLLABORATOR, "collaborator"),
    ]
    user = models.ForeignKey(Profile, on_delete=models.CASCADE, null=True)
    project = models.ForeignKey(Project, on_delete=models.CASCADE, null=True)
    role = models.CharField(
        max_length=13,
        choices=STATUS,
        default=ADMINISTRATOR,
    )

    def __str__(self):
        return str(self.project) + "_" + str(self.user)


class Tool(models.Model):
    name = models.CharField(max_length=100, blank=True, null=True)
    connector = models.CharField(max_length=100, blank=True, null=True)
    author = models.ForeignKey(Profile, on_delete=models.CASCADE, null=True)
    date_created = models.DateTimeField(blank=True, null=True)
    project = models.ForeignKey(Project, on_delete=models.CASCADE, null=True)

    def __str__(self):
        return str(self.id)

    def get_param(self, key):
        value = Toolparam.objects.values_list("value", flat=True).get(
            key=key, tool_id_id=str(self.id)
        )
        return value

    def get_connector(self):
        return get_connector(self)

    @property
    def tag_tool(self):
        connector = self.get_connector()
        tool_tag = connector.get_name()
        return tool_tag


class Toolparam(models.Model):
    tool_id = models.ForeignKey(Tool, on_delete=models.CASCADE, null=True)
    key = models.CharField(max_length=100, blank=True, null=True)
    value = models.CharField(max_length=2000, blank=True, null=True)

    def __str__(self):
        return "%s %s" % (self.key, self.tool_id)


class ToolparamChoiceField(ModelChoiceField):
    def label_from_instance(self, obj):
        return obj.value


class Mapping(models.Model):
    name = models.CharField(max_length=100, blank=True)
    tool_id = models.ForeignKey(Tool, on_delete=models.CASCADE, blank=True, null=True)
    type = models.CharField(max_length=100)
    object_id = models.CharField(max_length=100, blank=True, null=True)
    size = models.BigIntegerField(blank=True, null=True)
    author = models.ForeignKey(Profile, on_delete=models.CASCADE, null=True)
    foreign_id_obj = models.ForeignKey(
        Mappableobject, on_delete=models.CASCADE, null=True
    )

    def __str__(self):
        return self.name

    @classmethod
    def create_instance(cls, openlink_object, data_object, tool, author):
        mapping_object = Mapping()
        mapping_object.name = str(data_object.name)
        mapping_object.tool_id = tool
        mapping_object.type = type(openlink_object).__name__.lower()
        mapping_object.object_id = str(data_object.id)
        if type(openlink_object) == Dataset:
            connector = tool.get_connector()
            mapping_object.size = connector.get_space_info([str(data_object.id)])
        mapping_object.author = author
        mapping_object.foreign_id_obj = openlink_object
        return mapping_object

    @property
    def link_name(self):
        tool_qs = get_object_or_404(Tool, id=self.tool_id.id)
        connector = tool_qs.get_connector()
        try:
            object_info = connector.get_object(self.object_id, self.type)
            return object_info["name"]
        except Exception:
            return None

    @property
    def link_url(self):
        tool_qs = get_object_or_404(Tool, id=self.tool_id.id)
        connector = tool_qs.get_connector()
        return connector.get_url_link_to_an_object(self.type, self.object_id)

    @property
    def link_tool(self):
        tool_qs = get_object_or_404(Tool, id=self.tool_id.id)
        tool_name = tool_qs.name
        return tool_name

    @property
    def tag_tool(self):
        tool_qs = get_object_or_404(Tool, id=self.tool_id.id)
        connector = tool_qs.get_connector()
        tool_tag = connector.get_name
        return tool_tag

    @property
    def connector_class(self):
        tool_qs = get_object_or_404(Tool, id=self.tool_id.id)
        return tool_qs.get_connector()

    @property
    def get_supported_types_plural(self):
        from openlink.core.views import items

        data_type = self.type
        connector = self.connector_class
        ressource = connector.get_supported_types()
        ressources = {}
        i = 0
        for key in ressource:
            list_res_plural = []
            if key == data_type:
                i = 1
            else:
                if i == 1:
                    for value in ressource[key]:
                        list_res_plural.append(connector.get_plural_form_of_type(value))
                    ressources[items.get_plural_form_of_type(key)] = list_res_plural
        return ressources

    @property
    def get_supported_types(self):
        return self.connector_class.get_supported_types()


class MappingParam(models.Model):
    mapping_id = models.ForeignKey(Mapping, on_delete=models.CASCADE, null=True)
    key = models.CharField(max_length=100, blank=True, null=True)
    value = models.CharField(max_length=100, blank=True, null=True)

    def __str__(self):
        return "%s %s" % (self.key, self.mapping_id)



from django.urls import path, register_converter
from openlink.core import views
from openlink.core.views import items, mapping, projects, publish, tools, users


class FloatConverter:
    regex = r"[\d\.\d]+"

    def to_python(self, value):
        return float(value)

    def to_url(self, value):
        return "{}".format(value)


register_converter(FloatConverter, "float")


app_name = "core"
urlpatterns = [
    path("", views.home, name="home"),
    path("profile/<username>/", views.get_user_profile, name="user-profile"),
    path("contact/", views.contact, name="contact"),
    # projects creation
    path("projects", projects.projects, name="projects"),
    path("projects/new", projects.new_project, name="projects-new"),
    path("projects/add", projects.add_project, name="projects-add"),
    # projects tools
    path(
        "projects/<int:project_id>/tools",
        tools.list_tools_project,
        name="tools-project",
    ),
    path(
        "projects/<int:project_id>/tools/new",
        tools.select_connector_project,
        name="select-connector-project",
    ),
    path(
        "projects/<int:project_id>/tools/<str:connector>",
        tools.add_tool_project,
        name="add-tool-project",
    ),
    path(
        "projects/<int:project_id>/tools/<int:tool_id>/edit_toolparam",
        tools.toolsparam_edit_project,
        name="toolsparam-edit-project",
    ),
    path(
        "projects/<int:project_id>/tools/<int:tool_id>/edit_tool",
        tools.ToolUpdate.as_view(),
        name="tools-edit-project",
    ),
    path(
        "projects/<int:project_id>/tool/<int:tool_id>",
        tools.get_tool_info_project,
        name="tool-project",
    ),
    path(
        "projects/<int:project_id>/delete/tool/<int:pk>",
        tools.delete_tool,
        name="delete_tool",
    ),
    path(
        "project/<int:project_id>/<str:data_type>/<int:pk>/choose_tool",
        tools.choose_tool,
        name="choose_tool_project",
    ),
    path(
        "project/<int:project_id>/investigations/<int:investigation_id>/<str:data_type>/<int:pk>/choose_tool",
        tools.choose_tool,
        name="choose_tool_project",
    ),
    # projects editing
    path(
        "projects/<int:project_id>/edit",
        items.ProjectUpdate.as_view(),
        name="edit-project",
    ),
    path(
        "projects/<int:project_id>/add_user_mapping",
        users.add_user_mapping,
        name="add_user_mapping",
    ),
    path(
        "projects/<int:project_id>/manage_user",
        mapping.mapping_project_user,
        name="mapping_project_user",
    ),
    path(
        "projects/<int:project_id>/delete_user_mapping/<str:pk>",
        mapping.delete_user_mapping,
        name="delete_user_mapping",
    ),
    path("projects/<int:project_id>/", projects.details, name="projects-details"),
    path(
        "projects/<int:project_id>/delete/<str:data_type>/<int:pk>",
        projects.delete,
        name="delete",
    ),
    path(
        "projects/<int:project_id>/add_investigation",
        items.add_investigation,
        name="add-investigation",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/edit",
        items.InvestigationUpdate.as_view(),
        name="edit-investigation",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/add_study",
        items.add_study,
        name="add-study",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/study/<int:study_id>/edit",
        items.StudyUpdate.as_view(),
        name="edit-study",
    ),
    # project investigation study assay
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/select_study_to_create_assay",
        items.select_study_to_create_assay,
        name="select-study-to-create-assay",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/study/<int:study_id>/add_assay",
        items.add_assay,
        name="add-assay",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/assay/<int:assay_id>/edit",
        items.AssayUpdate.as_view(),
        name="edit-assay",
    ),
    # project investigation study assay dataset
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/select_assay_to_create_dataset",
        items.select_assay_to_create_dataset,
        name="select-assay-to-create-dataset",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/assay/<int:assay_id>/add_dataset",
        items.add_dataset,
        name="add-dataset",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/dataset/<int:dataset_id>/edit",
        items.DatasetUpdate.as_view(),
        name="edit-dataset",
    ),
    # project investigation map_id item
    # path(
    #     "projects/<int:project_id>/investigations/<int:investigation_id>/<str:map_id>/datasets",
    #     mapping.datasets,
    #     name="datasets",
    # ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/dataset/<int:dataset_id>/<int:tool_id>/<path:map_id>/datasets",
        mapping.datasets,
        name="datasets",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/assay/<int:assay_id>/<int:tool_id>/<path:map_id>/datasets",
        mapping.datasets,
        name="datasets",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/<int:tool_id>/<str:map_id>/datasets",
        mapping.datasets,
        name="datasets",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/study/<int:study_id>/<int:tool_id>/<str:map_id>/assays",
        mapping.assays,
        name="assays",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/<int:tool_id>/<str:map_id>/assays",
        mapping.assays,
        name="assays",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/<int:tool_id>/<str:map_id>/studies",
        mapping.studies,
        name="studies",
    ),
    # project investigation map_id select mapping
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/<str:map_id>/select_dataset_mapping",
        mapping.select_dataset_mapping,
        name="select_dataset_mapping",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/<int:tool_id>/<str:map_id>/select_assay_mapping",
        mapping.select_assay_mapping,
        name="select_assay_mapping",
    ),
    # publish
    path(
        "project/<int:project_id>/<str:data_type>/<int:pk>/choose_tool_for_publish",
        tools.choose_tool_for_publish,
        name="choose_tool_for_publish_project",
    ),
    path(
        "project/<int:project_id>/investigations/<int:investigation_id>/<str:data_type>/<int:pk>/choose_tool_for_publish",
        tools.choose_tool_for_publish,
        name="choose_tool_for_publish_project",
    ),
    path(
        "projects/<int:project_id>/<str:data_type>/<int:pk>/<str:tool>/<int:tool_id>/choose_object_to_publish",
        publish.choose_object_to_publish,
        name="choose_object_to_publish",
    ),
    path(
        "projects/<int:project_id>/<str:data_type>/<int:pk>/<str:tool>/<int:tool_id>/update/<path:saved_doi>/choose_object_to_publish",
        publish.choose_object_to_publish,
        name="choose_object_to_publish",
    ),
    path(
        "projects/<int:project_id>/<str:data_type>/<int:pk>/<str:tool>/<int:tool_id>/update/<path:saved_doi>/choose_metadata",
        publish.choose_metadata,
        name="choose_metadata",
    ),
    # testé quelle route est uttilisé pour choose_meta
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/<str:data_type>/<int:pk>/choose_metadata",
        publish.choose_metadata,
        name="choose_metadata",
    ),
    path(
        "projects/<int:project_id>/<str:data_type>/<int:pk>/<str:tool>/<int:tool_id>/choose_metadata",
        publish.choose_metadata,
        name="choose_metadata",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/<str:data_type>/<int:pk>/<str:tool>/<int:tool_id>/choose_metadata",
        publish.choose_metadata,
        name="choose_metadata",
    ),
    # path(
    #     "projects/<int:project_id>/investigations/<int:investigation_id>/<str:data_type>/<int:pk>/choose_non_linked_object",
    #     mapping.choose_non_linked_object,
    #     name="choose_non_linked_object",
    # ),
    path(
        "projects/<int:project_id>/<str:data_type>/<int:pk>/<int:tool_id>/choose_non_linked_object",
        mapping.choose_non_linked_object_tool,
        name="choose_non_linked_object_tool",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/<str:data_type>/<int:pk>/<int:tool_id>/choose_non_linked_object",
        mapping.choose_non_linked_object_tool,
        name="choose_non_linked_object_tool",
    ),
    path(
        "projects/<int:project_id>/<str:data_type>/<int:pk>/<str:tool>/<int:tool_id>/choose_non_linked_object",
        mapping.choose_non_linked_object_tool,
        name="choose_non_linked_object_tool",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/<str:data_type>/<int:pk>/<str:tool>/<int:tool_id>/choose_non_linked_object",
        mapping.choose_non_linked_object_tool,
        name="choose_non_linked_object_tool",
    ),
    # path('projects/<int:id>/investigations/<int:investigation_id>/<str:data_type>/<int:pk>/choose_object_to_publish', projects.choose_object_to_publish, name='choose_object_to_publish'),
    # path('projects/<int:id>/investigations/<int:investigation_id>/<str:data_type>/<int:pk>/<str:tool>/<int:tool_id>/choose_object_to_publish', projects.choose_object_to_publish, name='choose_object_to_publish'),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/<str:data_type>/<int:pk>/<int:tool_id>/<path:map_id>/choose_option_mapping",
        mapping.choose_option_mapping,
        name="choose_option_mapping",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/<str:data_type>/<int:tool_id>/<path:map_id>/choose_option_mapping",
        mapping.choose_option_mapping,
        name="choose_option_mapping",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/<int:tool_id>/<path:map_id>/choose_option_mapping",
        mapping.choose_option_mapping,
        name="choose_option_mapping",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/delete/<str:data_type>/<int:pk>",
        projects.delete,
        name="delete",
    ),
    path(
        "projects/<int:project_id>/investigations/<int:investigation_id>/delete/mapping/<str:data_type>/<int:pk>",
        mapping.delete_mapping,
        name="delete-mapping",
    ),
    path(
        "tool/<int:tool_id>/<str:data_type>/<path:map_id>",
        items.get_json_object,
        name="get_json_object",
    ),
    # projects/<int:project_id>/investigations/<int:investigation_id>/assay/<int:assay_id>/<path:map_id>/datasets
]



# Generated by Django 3.0.7 on 2022-01-26 13:30

import ckeditor.fields
import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Mappableobject",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Mapping",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(blank=True, max_length=100)),
                ("type", models.CharField(max_length=100)),
                ("object_id", models.CharField(blank=True, max_length=100, null=True)),
                ("size", models.BigIntegerField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name="Profile",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Team",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(blank=True, max_length=100)),
                (
                    "date",
                    models.DateTimeField(
                        blank=True, default=datetime.datetime.now, null=True
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Tool",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(blank=True, max_length=100, null=True)),
                ("connector", models.CharField(blank=True, max_length=100, null=True)),
                ("date_created", models.DateTimeField(blank=True, null=True)),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.Profile",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Assay",
            fields=[
                (
                    "mappableobject_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="core.Mappableobject",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("description", ckeditor.fields.RichTextField(blank=True, null=True)),
                ("date_created", models.DateTimeField(blank=True, null=True)),
                ("date_modified", models.DateTimeField(blank=True, null=True)),
            ],
            bases=("core.mappableobject",),
        ),
        migrations.CreateModel(
            name="Investigation",
            fields=[
                (
                    "mappableobject_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="core.Mappableobject",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("description", ckeditor.fields.RichTextField(blank=True, null=True)),
                ("date_created", models.DateTimeField(blank=True, null=True)),
                ("date_modified", models.DateTimeField(blank=True, null=True)),
            ],
            bases=("core.mappableobject",),
        ),
        migrations.CreateModel(
            name="Project",
            fields=[
                (
                    "mappableobject_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="core.Mappableobject",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("description", ckeditor.fields.RichTextField(blank=True, null=True)),
                ("date_created", models.DateTimeField(blank=True, null=True)),
                ("date_modified", models.DateTimeField(blank=True, null=True)),
            ],
            options={
                "permissions": (("manage_user", "Manage user"),),
            },
            bases=("core.mappableobject",),
        ),
        migrations.CreateModel(
            name="Toolparam",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("key", models.CharField(blank=True, max_length=100, null=True)),
                ("value", models.CharField(blank=True, max_length=2000, null=True)),
                (
                    "tool_id",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.Tool",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="profile",
            name="team",
            field=models.ManyToManyField(blank=True, to="core.Team"),
        ),
        migrations.AddField(
            model_name="profile",
            name="user",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
            ),
        ),
        migrations.CreateModel(
            name="MappingParam",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("key", models.CharField(blank=True, max_length=100, null=True)),
                ("value", models.CharField(blank=True, max_length=100, null=True)),
                (
                    "mapping_id",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.Mapping",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="mapping",
            name="author",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.Profile",
            ),
        ),
        migrations.AddField(
            model_name="mapping",
            name="foreign_id_obj",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.Mappableobject",
            ),
        ),
        migrations.AddField(
            model_name="mapping",
            name="tool_id",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.Tool",
            ),
        ),
        migrations.AddField(
            model_name="tool",
            name="project",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.Project",
            ),
        ),
        migrations.CreateModel(
            name="Study",
            fields=[
                (
                    "mappableobject_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="core.Mappableobject",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("description", ckeditor.fields.RichTextField(blank=True, null=True)),
                ("date_created", models.DateTimeField(blank=True, null=True)),
                ("date_modified", models.DateTimeField(blank=True, null=True)),
                ("assays", models.ManyToManyField(blank=True, to="core.Assay")),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="studies",
                        to="core.Profile",
                    ),
                ),
            ],
            bases=("core.mappableobject",),
        ),
        migrations.AddField(
            model_name="project",
            name="author",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.Profile",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="investigations",
            field=models.ManyToManyField(blank=True, to="core.Investigation"),
        ),
        migrations.CreateModel(
            name="MappingProjectUser",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "role",
                    models.CharField(
                        choices=[
                            ("administrator", "administrator"),
                            ("contributor", "contributor"),
                            ("collaborator", "collaborator"),
                        ],
                        default="administrator",
                        max_length=13,
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.Profile",
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.Project",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="investigation",
            name="author",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.Profile",
            ),
        ),
        migrations.AddField(
            model_name="investigation",
            name="studies",
            field=models.ManyToManyField(blank=True, to="core.Study"),
        ),
        migrations.CreateModel(
            name="Image",
            fields=[
                (
                    "mappableobject_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="core.Mappableobject",
                    ),
                ),
                ("name", models.CharField(blank=True, max_length=100)),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.Profile",
                    ),
                ),
            ],
            bases=("core.mappableobject",),
        ),
        migrations.CreateModel(
            name="Dataset",
            fields=[
                (
                    "mappableobject_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="core.Mappableobject",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("description", ckeditor.fields.RichTextField(blank=True, null=True)),
                ("date_created", models.DateTimeField(blank=True, null=True)),
                ("date_modified", models.DateTimeField(blank=True, null=True)),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.Profile",
                    ),
                ),
            ],
            bases=("core.mappableobject",),
        ),
        migrations.AddField(
            model_name="assay",
            name="author",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.Profile",
            ),
        ),
        migrations.AddField(
            model_name="assay",
            name="datasets",
            field=models.ManyToManyField(blank=True, to="core.Dataset"),
        ),
    ]



import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.urls import reverse
from openlink.core.forms import AddProjectUserForm
from openlink.core.models import MappingProjectUser, Project

logger = logging.getLogger(__name__)


@login_required
def get_access_level(request, project_id):
    # return current user access level for a given project
    access_level = ""
    user = request.user
    try:
        projects_list_user = Project.objects.get(
            author__user=request.user, id=project_id
        )
    except Project.DoesNotExist:
        projects_list_user = None
    if projects_list_user:
        access_level = str("administrator")
    proj_map = (
        MappingProjectUser.objects.filter(user__user=user)
        .filter(project__id=project_id)
        .values()
    )
    if proj_map.exists():
        for value in proj_map:
            access_level = value["role"]
    return access_level


@login_required
def add_user_mapping(request, *args, **kwargs):
    # Select a user to be map to the current project
    current_user = request.user
    current_proj = kwargs["project_id"]
    data_type = "project"
    mpu_list = MappingProjectUser.objects.filter(project__id=current_proj)
    project = get_object_or_404(Project, id=current_proj)

    if request.method == "POST":
        form = AddProjectUserForm(request.POST)
        user_mapping = form.save(commit=False)
        user_mapping.name = str(current_proj) + "_" + str(form.cleaned_data.get("user"))
        user_mapping.project = project
        user_mapping.save()
        messages.info(request, "User " + str(form.cleaned_data.get("user")) + " added")

        return HttpResponseRedirect(
            reverse("core:add_user_mapping", args=[current_proj])
        )

    else:
        form = AddProjectUserForm()

    return render(
        request,
        "mapping/add_user_mapping.html",
        {
            "form": form,
            "current_proj": current_proj,
            "data_type": data_type,
            "current_user": current_user,
            "data_type": data_type,
            "mpu_list": mpu_list,
            "project": project,
        },
    )



import json
import logging
import urllib

import openlink.core.connector
from django.apps import apps
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.forms import inlineformset_factory
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.urls import reverse
from django.utils import timezone
from guardian.decorators import permission_required_or_403
from guardian.shortcuts import assign_perm, remove_perm
from openlink.core.connector import ContainerDataObject, ContainerObject, Publisher
from openlink.core.forms import (
    LinkMappingForm,
    LinkMappingUserForm,
    SelectAssaysForm,
    SelectAssaysTool,
    SelectDatasetsTool,
    SelectMultipleObjectOption,
    SelectObjectOption,
    SelectObjectTool,
    SelectStudiesForm,
    SelectStudiesTool,
)
from openlink.core.lib import utils
from openlink.core.models import (
    Assay,
    Investigation,
    Mapping,
    MappingProjectUser,
    Profile,
    Project,
    Study,
    Tool,
)
from openlink.core.views import items

logger = logging.getLogger(__name__)


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def studies(request, *args, **kwargs):
    # Manage on-the-fly studies mapping
    # reference_data_type = request.session["reference_data_type"]
    project_id = kwargs["project_id"]
    investigation_id = kwargs["investigation_id"]
    tool_id = kwargs["tool_id"]
    tool = get_object_or_404(Tool, id=tool_id)
    map_id = kwargs["map_id"]
    data_type = "study"
    unlink_studies = []
    project = get_object_or_404(Project, author__user=request.user, pk=project_id)
    investigation = get_object_or_404(
        Investigation, author__user=request.user, pk=investigation_id
    )
    tools_qs = Tool.objects.filter(
        project=project,
        mapping__foreign_id_obj__investigation__id=investigation_id,
        mapping__object_id=map_id,
    )
    map_instance = Mapping.objects.get(
        foreign_id_obj__investigation__id=investigation_id, object_id=map_id
    )
    # Search for studies not linked to the current Openlink investigation
    for tool_qs in tools_qs:
        ms_list = Mapping.objects.filter(
            foreign_id_obj__study__investigation__id=investigation_id, tool_id=tool_qs
        )
        linked_studies = []
        for mapped_studies in ms_list:
            linked_studies.append(str(mapped_studies.object_id))
        connector = tool_qs.get_connector()
        reference_mapping = None
        container_mapping = ContainerObject(
            map_id, map_instance.name, map_instance.type
        )
        connector_studies = connector.get_data_objects(
            data_type, reference_mapping, container_mapping
        )
        for connector_study in connector_studies:
            if str(connector_study.id) not in linked_studies:
                unlink_studies.append(connector_study)

    studies = []
    if unlink_studies is not None:
        for study in unlink_studies:
            study = (str(study.id), study.name)
            studies.append(study)

    if request.method == "POST":
        list_studies = request.POST.getlist("study_name")
        objects_id = "%s" % (",".join(map(str, list_studies)))
        if len(list_studies) == 0:
            messages.info(request, "Please select at least one project!")
            form = SelectStudiesTool(studies)
            pass
        else:
            request.session["map_children"] = True
            request.session["data_type"] = data_type
            request.session["map_id"] = objects_id
            request.session["parent_id"] = investigation.id
            request.session["parent_type"] = "investigation"
            request.session["reference_data_type"] = "investigation"
            request.session["reference_data_id"] = investigation.id
            return HttpResponseRedirect(
                reverse(
                    "core:choose_option_mapping",
                    args=[project.id, investigation_id, data_type, tool.id, map_id],
                )
            )
    else:
        form = SelectStudiesTool(studies)
    return render(
        request,
        "mapping/item_list/studies_list.html",
        {
            "form": form,
            "project": project,
            "investigation": investigation,
            "unlink_studies": unlink_studies,
        },
    )


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def assays(request, *args, **kwargs):
    # Manage on-the-fly assays mapping
    # reference_data_type = request.session["reference_data_type"]
    project_id = kwargs["project_id"]
    investigation_id = kwargs["investigation_id"]
    map_id = kwargs["map_id"]
    unlink_assays = []
    data_type = "assay"
    structure = "list"
    json_tree = []
    project = get_object_or_404(Project, pk=project_id)
    investigation = get_object_or_404(Investigation, pk=investigation_id)
    if "study_id" in kwargs:
        tools_qs = Tool.objects.filter(
            mapping__foreign_id_obj__study__investigation__id=investigation_id,
            mapping__object_id=map_id,
        )
    else:
        tools_qs = Tool.objects.filter(
            mapping__foreign_id_obj__investigation__id=investigation_id,
            mapping__object_id=map_id,
        )
    # Search for assays not linked to the current Openlink investigation
    for tool_qs in tools_qs:
        ma_list = Mapping.objects.filter(
            foreign_id_obj__assay__study__investigation__id=investigation_id,
            tool_id=tool_qs,
        )
        linked_assays = []
        for mapped_assays in ma_list:
            linked_assays.append(str(mapped_assays.object_id))
        if "study_id" in kwargs:
            reference_mapping = Mapping.objects.get(
                foreign_id_obj__investigation__id=investigation_id, tool_id=tool_qs
            )
            container_mapping = Mapping.objects.get(
                foreign_id_obj__study__investigation__id=investigation_id,
                object_id=map_id,
                tool_id=tool_qs,
            )
            container_instance = ContainerObject(
                container_mapping.object_id,
                container_mapping.name,
                container_mapping.type,
            )
        else:
            reference_mapping = Mapping.objects.get(
                foreign_id_obj__investigation__id=investigation_id,
                object_id=map_id,
                tool_id=tool_qs,
            )
            container_instance = None
        reference_instance = ContainerObject(
            reference_mapping.object_id, reference_mapping.name, reference_mapping.type
        )
        connector = tool_qs.get_connector()
        connector_assays = connector.get_data_objects(
            "assay", reference_instance, container_instance
        )
        if connector.get_data_structure() == openlink.core.connector.TREE_STRUCTURE:
            structure = "tree"
            # Initiate tree structure
            investigation_info = connector.get_investigation(map_id)
            json_tree.append(
                {
                    "id": investigation_info.id,
                    "parent": "#",
                    "text": investigation_info.name,
                    "icon": "fa fa-inbox",
                    "state": {"checkbox_disabled": True},
                }
            )
        for connector_assay in connector_assays:
            if str(connector_assay.id) not in linked_assays:
                unlink_assays.append(connector_assay)
        tool = tool_qs
    assays = []
    if unlink_assays is not None:
        for assay in unlink_assays:
            assay = (str(assay.id), assay.name)
            assays.append(assay)
    # Create and link assays in selected study
    if request.method == "POST":
        list_assays = request.POST.getlist("assay_name")
        str_list_assays = ", ".join(list_assays)

        if "study_id" in kwargs:
            if len(list_assays) == 0:
                messages.info(request, "Please select at least one assay!")
                form = SelectAssaysTool(assays)
                pass
            else:
                request.session["map_children"] = True
                request.session["data_type"] = data_type
                request.session["map_id"] = str_list_assays
                request.session["parent_id"] = kwargs["study_id"]
                request.session["parent_type"] = "study"
                request.session["reference_data_type"] = "study"
                request.session["reference_data_id"] = kwargs["study_id"]
                return HttpResponseRedirect(
                    reverse(
                        "core:choose_option_mapping",
                        args=[
                            project.id,
                            investigation_id,
                            data_type,
                            kwargs["study_id"],
                            tool.id,
                            map_id,
                        ],
                    )
                )
        else:
            # Display a form to select a study where assays will be created
            form = SelectStudiesForm(investigation=investigation)
            if Study.objects.filter(investigation__id=investigation_id).exists():
                objects_to_select = True
            else:
                objects_to_select = False

            request.session["map_id"] = str_list_assays
            request.session["reference_data_type"] = "investigation"
            request.session["reference_data_id"] = investigation_id

            return render(
                request,
                "mapping/form_select_item.html",
                {
                    "form": form,
                    "project": project,
                    "investigation": investigation,
                    "selected_items": str_list_assays,
                    "map_id": map_id,
                    "type_to_select": "study",
                    "data_type": "assay",
                    "tool_id": tool.id,
                    "tool_name": connector.get_name,
                    "function": "select_assay_mapping",
                    "objects_to_select": objects_to_select,
                },
            )
    else:
        form = SelectAssaysTool(assays)

        if structure == "list":
            return render(
                request,
                "mapping/item_list/assays_list.html",
                {
                    "form": form,
                    "project": project,
                    "investigation": investigation,
                    "unlink_assay": unlink_assays,
                },
            )
        else:
            json_tree = json.dumps(json_tree)
            return render(
                request,
                "mapping/item_list/datasets_list.html",
                {
                    "form": form,
                    "project": project,
                    "investigation": investigation,
                    "id": project.id,
                    "json_tree": json_tree,
                    "data_type": "assay",
                    "structure": structure,
                    "tool": tool,
                },
            )


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def select_assay_mapping(request, *args, **kwargs):
    # Display study from the complete project page
    project_id = kwargs["project_id"]
    investigation_id = kwargs["investigation_id"]
    map_id = kwargs["map_id"]
    project = get_object_or_404(Project, pk=project_id)
    data_type = "assay"
    tools_qs = Tool.objects.filter(
        project=project,
        mapping__foreign_id_obj__investigation__id=investigation_id,
        mapping__object_id=map_id,
    )
    tool = tools_qs[0]

    if request.method == "POST":
        selected_study = request.POST.get("studies_name")
        selected_assays_string = request.POST.get("assay")
        map_id = str(selected_assays_string).strip("'<>() ").replace("'", '"')

        request.session["data_type"] = data_type
        request.session["map_id"] = selected_assays_string
        request.session["parent_id"] = selected_study
        request.session["map_children"] = True
        return HttpResponseRedirect(
            reverse(
                "core:choose_option_mapping",
                args=[project.id, investigation_id, data_type, tool.id, map_id],
            )
        )


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def datasets(request, *args, **kwargs):
    # reference_data_type = request.session["reference_data_type"]
    # Manage on-the-fly datasets mapping
    project_id = kwargs["project_id"]
    investigation_id = kwargs["investigation_id"]
    project = get_object_or_404(Project, pk=project_id)
    investigation = get_object_or_404(Investigation, pk=investigation_id)
    map_id = kwargs["map_id"]
    linked_datasets = []
    unlink_datasets = []
    data_type = "dataset"
    structure = "list"
    json_tree = []
    tool = Tool.objects.filter(project=project, mapping__object_id=map_id).first()
    connector = tool.get_connector()
    # Search for datasets not linked to the current Openlink investigation
    if "assay_id" in kwargs:
        if tool is None:
            map_id = urllib.parse.quote(map_id)
            tool = Tool.objects.filter(
                project=project, mapping__object_id=map_id
            ).first()
        assay_id = kwargs["assay_id"]
        assay = get_object_or_404(Assay, id=assay_id)
        mapping_assay = Mapping.objects.filter(
            foreign_id_obj__assay__id=assay.id, tool_id__connector=tool.connector
        )
        for map_assay in mapping_assay:
            map_assay_object = ContainerDataObject(
                map_assay.object_id, map_assay.name, map_assay.type
            )
            objects = connector.get_data_objects(data_type, None, map_assay_object)
            if connector.get_data_structure() == openlink.core.connector.TREE_STRUCTURE:
                structure = "tree"
                assay_info = connector.get_data_object("assay", map_assay.object_id)
                json_tree.append(
                    {
                        "id": assay_info.id,
                        "parent": "#",
                        "text": assay_info.name,
                        "state": {"checkbox_disabled": True},
                    }
                )
        mo_list = Mapping.objects.filter(
            foreign_id_obj__dataset__assay__id=assay.id,
            tool_id__connector=tool.connector,
        )
        for mapping_object in mo_list:
            linked_datasets.append((str(mapping_object.object_id)))
        for connector_object in objects:
            if str(connector_object.id) not in linked_datasets:
                if connector_object not in unlink_datasets:
                    unlink_datasets.append(connector_object)
        datasets = []
        if unlink_datasets is not None:
            for dataset in unlink_datasets:
                dataset = (str(dataset.id), dataset.name)
                datasets.append(dataset)
    elif "study_id" in kwargs:
        if tool is None:
            map_id = urllib.parse.quote(map_id)
            tool = Tool.objects.filter(
                project=project, mapping__object_id=map_id
            ).first()
        study_id = kwargs["study_id"]
        study = get_object_or_404(Study, id=study_id)
        mapping_study = Mapping.objects.filter(
            foreign_id_obj__assay__study__id=study.id, tool_id__connector=tool.connector
        )
        for map_study in mapping_study:
            map_study_object = ContainerDataObject(
                map_study.object_id, map_study.name, map_study.type
            )
            objects = connector.get_data_objects(data_type, None, map_study_object)
            if connector.get_data_structure() == openlink.core.connector.TREE_STRUCTURE:
                structure = "tree"
                study_info = connector.get_data_object("study", map_study.object_id)
                json_tree.append(
                    {
                        "id": study_info.id,
                        "parent": "#",
                        "text": study_info.name,
                        "state": {"checkbox_disabled": True},
                    }
                )
        mo_list = Mapping.objects.filter(
            foreign_id_obj__dataset__assay__study__id=study.id,
            tool_id__connector=tool.connector,
        )
        for mapping_object in mo_list:
            linked_datasets.append((str(mapping_object.object_id)))
        for connector_object in objects:
            if str(connector_object.id) not in linked_datasets:
                if connector_object not in unlink_datasets:
                    unlink_datasets.append(connector_object)
        datasets = []
        if unlink_datasets is not None:
            for dataset in unlink_datasets:
                dataset = (str(dataset.id), dataset.name)
                datasets.append(dataset)
    else:
        tools_qs = Tool.objects.filter(
            project=project,
            mapping__foreign_id_obj__investigation__id=investigation_id,
            mapping__object_id=map_id,
        )
        for tool_qs in tools_qs:
            md_list = Mapping.objects.filter(
                foreign_id_obj__dataset__assay__study__investigation__id=investigation_id,
                tool_id=tool_qs,
            )
            linked_datasets = []
            for mapped_datasets in md_list:
                linked_datasets.append(str(mapped_datasets.object_id))
            connector = tool_qs.get_connector()
            connector_datasets = connector.get_datasets(map_id)
            for connector_dataset in connector_datasets:
                if str(connector_dataset.id) not in linked_datasets:
                    unlink_datasets.append(connector_dataset)
        datasets = []
        if unlink_datasets is not None:
            for dataset in unlink_datasets:
                obj_id = dataset.id
                obj = dataset.name
                dataset = (str(obj_id), obj)
                datasets.append(dataset)
        if connector.get_data_structure() == openlink.core.connector.TREE_STRUCTURE:
            structure = "tree"
            investigation_info = connector.get_investigation(map_id)
            json_tree.append(
                {
                    "id": investigation_info.id,
                    "parent": "#",
                    "text": investigation_info.name,
                    "icon": "fa fa-inbox",
                    "state": {"checkbox_disabled": True},
                }
            )
    # Create and link datasets in selected assay
    if request.method == "POST":
        list_datasets = request.POST.getlist("dataset_name")
        str_list_datasets = (
            str(list_datasets)
            .strip("'<>() ")
            .replace("'", '"')
            .replace('["[', "[")
            .replace(']"]', "]")
        )
        selected_datasets = json.loads(str_list_datasets)
        str_list_datasets_final = ", ".join(selected_datasets)
        # If assay is already known
        if "assay_id" in kwargs:
            # selected_datasets = json.loads(dataform)
            # selected_assay = assay_id
            if len(list_datasets) == 0:
                messages.info(request, "Please select at least one dataset!")
                form = SelectDatasetsTool(datasets)
                pass
            else:
                request.session["map_children"] = True
                request.session["data_type"] = data_type
                request.session["map_id"] = str_list_datasets_final
                request.session["parent_id"] = kwargs["assay_id"]
                request.session["parent_type"] = "assay"
                request.session["reference_data_type"] = "assay"
                request.session["reference_data_id"] = kwargs["assay_id"]
                return HttpResponseRedirect(
                    reverse(
                        "core:choose_option_mapping",
                        args=[project.id, investigation_id, data_type, tool.id, map_id],
                    )
                )
        else:
            request.session["map_id"] = str_list_datasets_final
            if "study_id" in kwargs:
                request.session["reference_data_type"] = "study"
                request.session["reference_data_id"] = kwargs["study_id"]
            else:
                request.session["reference_data_type"] = "investigation"
                request.session["reference_data_id"] = kwargs["investigation_id"]
            if len(list_datasets) == 0:
                messages.info(request, "Please select at least one dataset!")
                form = SelectDatasetsTool(datasets)
                pass
            else:
                # Display a form to select a study where assays will be created
                form = SelectAssaysForm(investigation=investigation)
                if Assay.objects.filter(
                    study__investigation__id=investigation_id
                ).exists():
                    objects_to_select = True
                else:
                    objects_to_select = False
                return render(
                    request,
                    "mapping/form_select_item.html",
                    {
                        "form": form,
                        "project": project,
                        "investigation": investigation,
                        "selected_items": list_datasets,
                        "map_id": map_id,
                        "type_to_select": "assay",
                        "data_type": "dataset",
                        "connector": connector,
                        "function": "select_dataset_mapping",
                        "objects_to_select": objects_to_select,
                    },
                )

    else:
        form = SelectDatasetsTool(datasets)
    json_tree = json.dumps(json_tree)
    return render(
        request,
        "mapping/item_list/datasets_list.html",
        {
            "form": form,
            "project": project,
            "investigation": investigation,
            "unlink_datasets": unlink_datasets,
            "id": project.id,
            "json_tree": json_tree,
            "data_type": data_type,
            "structure": structure,
            "tool": tool,
        },
    )


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def select_dataset_mapping(request, *args, **kwargs):
    # Display studies and assay as in the complete project page
    project_id = kwargs["project_id"]
    investigation_id = kwargs["investigation_id"]
    map_id = kwargs["map_id"]
    project = get_object_or_404(Project, pk=project_id)
    tools_qs = Tool.objects.filter(
        project=project,
        mapping__foreign_id_obj__investigation__id=investigation_id,
        mapping__object_id=map_id,
    )
    tool = tools_qs[0]

    if request.method == "POST":
        selected_assay = request.POST.get("assays_name")
        selected_datasets_string = request.POST.get("dataset")
        dataform = str(selected_datasets_string).strip("'<>() ").replace("'", '"')
        selected_datasets = json.loads(dataform)

        request.session["data_type"] = "dataset"
        request.session["map_id"] = selected_datasets
        request.session["parent_id"] = selected_assay
        request.session["map_children"] = True
        return HttpResponseRedirect(
            reverse(
                "core:choose_option_mapping",
                args=[project.id, investigation_id, "dataset", tool.id, map_id],
            )
        )


def get_objects_linked_in_project(data_type, reference_project, tool):
    # Get a list of objects id already linked to an Openlink project
    linked_objects = []
    if data_type == "study":
        mo_list = Mapping.objects.filter(
            foreign_id_obj__study__investigation__project__id=reference_project,
            tool_id=tool,
        )
    elif data_type == "assay":
        mo_list = Mapping.objects.filter(
            foreign_id_obj__assay__study__investigation__project__id=reference_project,
            tool_id=tool,
        )
    elif data_type == "dataset":
        mo_list = Mapping.objects.filter(
            foreign_id_obj__dataset__assay__study__investigation__project__id=reference_project,
            tool_id=tool,
        )
    else:
        if data_type == "investigation":
            mo_list = Mapping.objects.filter(
                foreign_id_obj__investigation__project__id=reference_project,
                tool_id=tool,
            )

    for mo in mo_list:
        linked_objects.append(mo.object_id)
    return linked_objects


def get_objects_linked_in_investigation(data_type, reference_investigation, tool):
    # Get a list of objects id already linked to an Openlink investigation
    linked_objects = []
    if data_type == "study":
        mo_list = Mapping.objects.filter(
            foreign_id_obj__study__investigation__id=reference_investigation,
            tool_id=tool,
        )
    elif data_type == "assay":
        mo_list = Mapping.objects.filter(
            foreign_id_obj__assay__study__investigation__id=reference_investigation,
            tool_id=tool,
        )
    elif data_type == "dataset":
        mo_list = Mapping.objects.filter(
            foreign_id_obj__dataset__assay__study__investigation__id=reference_investigation,
            tool_id=tool,
        )
    for mo in mo_list:
        linked_objects.append(mo.object_id)
    return linked_objects


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def choose_non_linked_object_tool(request, *args, **kwargs):
    # Display a selection of available objects to link
    project = get_object_or_404(Project, id=kwargs["project_id"])
    tool_qs = get_object_or_404(Tool, project=project, id=kwargs["tool_id"])
    if "investigation_id" in kwargs:
        current_inv = kwargs["investigation_id"]
        reference_item = get_object_or_404(
            Investigation, project=project, id=current_inv
        )
        if Mapping.objects.filter(
            foreign_id_obj=reference_item, tool_id=tool_qs
        ).exists():
            reference_mapping = Mapping.objects.get(
                foreign_id_obj=reference_item, tool_id=tool_qs
            )
            reference_container = ContainerDataObject(
                reference_mapping.object_id,
                reference_mapping.name,
                reference_mapping.type,
            )
        else:
            reference_container = None
    else:
        reference_container = None
    pk = kwargs["pk"]
    data_type = kwargs["data_type"]
    request.session["reference_data_type"] = data_type
    request.session["reference_data_id"] = pk
    tool_id = kwargs["tool_id"]
    unlinked_objects = []
    parent_objects = []
    json_tree = []
    connector = tool_qs.get_connector()
    list_ressource = connector.get_supported_types()
    Model = apps.get_model("core", data_type)
    for item in list_ressource:
        if item == data_type:
            data_type_map = list_ressource[item]

    # Get a list of objects already linked to Openlink
    if data_type in list_ressource:
        linked_objects = get_objects_linked_in_project(data_type, project.id, tool_qs)

    # Get non linked DataObject and initiate json for tree structure
    item_parent = Model.get_parent(pk)[0]
    if Mapping.objects.filter(foreign_id_obj=item_parent, tool_id=tool_qs).exists():
        mapping_parent = Mapping.objects.get(
            foreign_id_obj=item_parent, tool_id=tool_qs
        )
        container_parent = ContainerDataObject(
            mapping_parent.object_id, mapping_parent.name, mapping_parent.type
        )
    else:
        container_parent = None

    for connector_object in connector.get_data_objects(
        data_type, reference_container, container_parent
    ):
        if str(connector_object.id) not in linked_objects:
            unlinked_objects.append(connector_object)

    if connector.get_data_structure() == openlink.core.connector.TREE_STRUCTURE:
        structure = "tree"
        if container_parent is not None:
            data_type_parent = Model.get_parent_type()
            parent_info = connector.get_data_object(
                str(data_type_parent.__name__).lower(), container_parent.id
            )
            json_tree.append(
                {
                    "id": parent_info.id,
                    "parent": "#",
                    "text": parent_info.name,
                    "state": {"checkbox_disabled": True},
                }
            )
        else:
            parents_info = connector.get_data_objects(
                data_type, reference_container, container_parent
            )
            for parent_info in parents_info:
                json_tree.append(
                    {
                        "id": parent_info.id,
                        "parent": "#",
                        "text": parent_info.name,
                        "state": {"checkbox_disabled": True},
                    }
                )
    else:
        structure = "list"

    if request.method == "POST":
        object_id = request.POST["object_name"]
        request.session["map_children"] = False
        request.session["data_type"] = data_type
        request.session["map_id"] = object_id
        print(object_id)
        for obj in unlinked_objects:
            if str(obj.id) == str(object_id):
                tool = tool_qs
                tool_id = tool.id

        # return to edit study URL
        if "investigation_id" not in kwargs:
            current_inv = pk
        return HttpResponseRedirect(
            reverse(
                "core:choose_option_mapping",
                args=[project.id, current_inv, data_type, pk, tool_id, object_id],
            )
        )

    else:
        form = SelectObjectTool(unlinked_objects)

    if "investigation_id" in kwargs:
        json_tree = json.dumps(json_tree)
        return render(
            request,
            "mapping/form_choose_object_mapping.html",
            {
                "form": form,
                "unlinked_objects": unlinked_objects,
                "current_proj": project.id,
                "current_inv": current_inv,
                "data_type": data_type,
                "data_type_map": data_type_map,
                "pk": pk,
                "function": "choose_non_linked_object",
                "project": project,
                "structure": structure,
                "parent_objects": parent_objects,
                "json_tree": json_tree,
                "tool": tool_id,
                "tool_name": tool_qs.name,
            },
        )
    else:
        return render(
            request,
            "mapping/form_choose_object_mapping.html",
            {
                "form": form,
                "unlinked_objects": unlinked_objects,
                "current_proj": project.id,
                "data_type": data_type,
                "data_type_map": data_type_map,
                "pk": pk,
                "function": "choose_non_linked_object",
                "project": project,
                "structure": structure,
                "parent_objects": parent_objects,
                "tool_name": tool_qs.name,
            },
        )


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def choose_option_mapping(request, *args, **kwargs):
    # Manage one item mapping and his options

    data_type = request.session["data_type"]
    author = get_object_or_404(Profile, user=request.user)
    tool = get_object_or_404(Tool, id=kwargs["tool_id"])
    connector = tool.get_connector()
    Model = apps.get_model("core", data_type)
    current_proj = kwargs["project_id"]
    reference_data_type = request.session["reference_data_type"]
    reference_data_id = request.session["reference_data_id"]
    project = get_object_or_404(Project, id=current_proj)
    if request.session["map_children"] is True:
        objects_id = request.session["map_id"].split(",")
        pk = request.session["parent_id"]
    else:
        pk = kwargs["pk"]
        objects_id = kwargs["map_id"].split(",")
        object_info = connector.get_data_object(data_type, objects_id[0])
        openlink_object = get_object_or_404(Model, id=kwargs["pk"])
    if "investigation_id" in kwargs:
        current_inv = kwargs["investigation_id"]
    list_ressource = connector.get_supported_types()
    reference_mapping = None

    # Get list of mapping type in plural form
    (
        data_type_map,
        list_objects,
        list_objects_map,
    ) = get_list_mapping_type_of_available_children(
        list_ressource, data_type, connector
    )

    if request.method == "POST":
        for object_id in objects_id:
            object_info = connector.get_data_object(data_type, object_id)

            # Get or Create Openlink object if it doesn't already exist
            if request.session["map_children"] is True:
                update_object = Model.create_instance(object_info, author)
                update_object.save()
                messages.info(request, str(update_object.name) + " created ")
                openlink_object = update_object
                parent_id = request.session["parent_id"]
                parent_type = Model.get_parent_type()
                parent_object = get_object_or_404(parent_type, id=parent_id)
                parent_type.link_child_to_parent(parent_object, openlink_object)
            else:
                update_object = get_object_or_404(Model, id=pk)

            # Update Openlink study name
            if "replace_openlink_name" in request.POST:
                name = utils.add_number_at_end_if_name_exist(
                    object_info.name, update_object
                )
                update_object.name = name
                update_object.date_modified = timezone.now()
                update_object.save()
                messages.info(
                    request,
                    data_type.capitalize() + " " + str(update_object.name) + " updated",
                )

            logger.debug(request.POST)
            if "use_description" in request.POST:
                update_object.description = object_info.description
                update_object.date_modified = timezone.now()
                update_object.save()
                messages.info(
                    request,
                    data_type.capitalize()
                    + " "
                    + str(update_object.name)
                    + " description updated",
                )
            elif "add_description" in request.POST:
                update_object.description = (
                    update_object.description + "\n" + object_info.description
                )
                update_object.date_modified = timezone.now()
                update_object.save()
                messages.info(
                    request,
                    data_type.capitalize()
                    + " "
                    + str(update_object.name)
                    + " description updated",
                )

            # Create Mapping for the new openlink object
            new_mapping_object = Mapping.create_instance(
                openlink_object, object_info, tool, author
            )
            new_mapping_object.save()
            messages.info(
                request, "Mapping " + str(new_mapping_object.name) + " created "
            )

            if "create_and_link_children" in request.POST:
                new_container_object = connector.get_data_object(
                    new_mapping_object.type, new_mapping_object.object_id
                )
                utils.create_children_mappings(
                    request,
                    tool,
                    openlink_object,
                    reference_mapping,
                    new_container_object,
                    author,
                )

            tool_option = {}
            for key, value in request.POST.items():
                if key.startswith(tool.name.lower()):
                    tool_option[key] = value
            if connector.has_mapping_options is True:
                connector.option_traitement(tool_option, new_mapping_object)
        # Return to edit openlink object page
        try:
            del request.session["parent_id"]
            del request.session["parent_type"]
            del request.session["reference_data_type"]
            del request.session["map_children"]
            del request.session["data_type"]
            del request.session["map_id"]
        except KeyError:
            pass
        if data_type == "investigation":
            return HttpResponseRedirect(
                reverse(
                    "core:edit-" + str(data_type),
                    args=[current_proj, kwargs["pk"]],
                )
            )
        else:
            if "reference_data_id" in request.session:
                del request.session["reference_data_id"]
                if current_inv == reference_data_id:
                    return HttpResponseRedirect(
                        reverse(
                            "core:edit-" + str(reference_data_type),
                            args=[current_proj, reference_data_id],
                        )
                    )
                else:
                    return HttpResponseRedirect(
                        reverse(
                            "core:edit-" + str(reference_data_type),
                            args=[current_proj, current_inv, reference_data_id],
                        )
                    )
            else:
                return HttpResponseRedirect(
                    reverse(
                        "core:edit-" + str(data_type),
                        args=[current_proj, current_inv, kwargs["pk"]],
                    )
                )

    else:
        has_mapping_options = tool.get_connector().has_mapping_options()
        if request.session["map_children"] is True:
            form = SelectMultipleObjectOption(
                data_type_map=data_type_map,
                has_mapping_options=has_mapping_options,
                tool_connector=connector,
                list_objects=list_objects,
                list_objects_map=list_objects_map,
                action="choose_option_mapping",
            )
            return render(
                request,
                "mapping/form_choose_option_mapping.html",
                {
                    "form": form,
                    "current_proj": current_proj,
                    "current_inv": current_inv,
                    "data_type": data_type,
                    "data_type_map": data_type_map,
                    "tool_name": tool.name,
                    "project": project,
                    "function": "choose_option_mapping",
                    "pk": pk,
                },
            )

        else:
            if "pk" in kwargs:
                form = SelectObjectOption(
                    data_type=data_type,
                    data_type_map=data_type_map,
                    object_name=object_info.name,
                    has_mapping_options=has_mapping_options,
                    tool_connector=connector,
                    object_desc=object_info.description,
                    list_objects=list_objects,
                    list_objects_map=list_objects_map,
                    action="choose_option_mapping",
                )
                return render(
                    request,
                    "mapping/form_choose_option_mapping.html",
                    {
                        "form": form,
                        "current_proj": current_proj,
                        "current_inv": current_inv,
                        "data_type": reference_data_type,
                        "pk": kwargs["pk"],
                        "data_type_map": data_type_map,
                        "tool_name": tool.name,
                        "project": project,
                        "object_desc": object_info.description,
                        "function": "choose_option_mapping",
                    },
                )


@login_required
@permission_required_or_403("change_project", (Project, "id", "project_id"))
def delete_mapping(request, *args, **kwargs):
    # Delete mapping instance from openlink
    project_id = kwargs["project_id"]
    investigation_id = kwargs["investigation_id"]
    obj_id = kwargs["pk"]
    data_type = kwargs["data_type"]
    obj_to_delete = get_object_or_404(Mapping, id=obj_id)
    logging.debug(obj_to_delete)
    type_to_delete = obj_to_delete.type
    type_id = obj_to_delete.foreign_id_obj.id
    if request.method == "POST":

        if data_type == "investigation":
            investigation = get_object_or_404(Investigation, id=investigation_id)
            tool_qs = get_object_or_404(Tool, mapping__id=obj_id)
            tool_id = tool_qs.id
            connector = tool_qs.get_connector()
            list_save = []
            list_items = items.get_items_id(investigation, list_save)
            object_id = get_object_or_404(Mapping, id=obj_id).object_id
            for item in list_items:
                try:
                    if issubclass(connector.__class__, Publisher):
                        mapping_query = Mapping.objects.get(
                            foreign_id_obj=item,
                            tool_id__id=tool_id,
                            object_id=object_id,
                        )
                    else:
                        mapping_query = Mapping.objects.get(
                            foreign_id_obj=item, tool_id__id=tool_id
                        )
                except Mapping.DoesNotExist:
                    mapping_query = None
                if mapping_query:
                    mapping_obj_to_delete = mapping_query
                    mapping_obj_to_delete.delete()
                    messages.info(request, str(mapping_obj_to_delete) + " deleted")

        obj_to_delete = get_object_or_404(Mapping, id=obj_id)
        logging.debug(obj_to_delete)
        obj_to_delete.delete()

        return HttpResponseRedirect(
            reverse(
                "core:edit-" + str(data_type),
                kwargs={
                    "project_id": project_id,
                    "investigation_id": investigation_id,
                    str(data_type) + "_id": type_id,
                },
            )
        )

    else:
        form = LinkMappingForm
    return render(
        request,
        "mapping/form_delete_mapping.html",
        {
            "form": form,
            "obj": obj_to_delete,
            "data_type": type_to_delete,
            "project_id": project_id,
            "investigation_id": investigation_id,
            "type_id": type_id,
        },
    )


def delete_user_mapping(request, *args, **kwargs):
    # Delete user and his mapping objects from a project
    project_id = kwargs["project_id"]
    project = get_object_or_404(Project, id=project_id)
    if request.method == "POST":
        users_to_delete = request.POST["users_list"]
        users_collection = users_to_delete.split(",")
        for user in users_collection:
            if user != "":
                user_to_delete = get_object_or_404(MappingProjectUser, id=user)
                username_to_delete = user_to_delete.user
                user_to_delete.delete()
                messages.info(
                    request, str(username_to_delete) + " deleted from " + str(project)
                )

                list_save = []
                list_items = items.get_items_id(project, list_save)
                for item in list_items:
                    try:
                        mapping_query = Mapping.objects.get(
                            foreign_id_obj=item, author=username_to_delete
                        )
                    except Mapping.DoesNotExist:
                        mapping_query = None
                    if mapping_query:
                        mapping_obj_to_delete = mapping_query
                        mapping_obj_to_delete.delete()
                        messages.info(request, str(mapping_obj_to_delete) + " deleted")

        return HttpResponseRedirect(
            reverse("core:mapping_project_user", args=[project_id])
        )


@login_required
@permission_required_or_403("manage_user", (Project, "id", "project_id"))
def mapping_project_user(request, *args, **kwargs):
    # Assign new user permissions to the current project
    current_user = request.user
    current_proj = kwargs["project_id"]
    data_type = "project"
    mpu_list = MappingProjectUser.objects.filter(project__id=current_proj)
    project = get_object_or_404(Project, id=current_proj)
    ManageProjectUserFormset = inlineformset_factory(
        Project,
        MappingProjectUser,
        fields=(
            "user",
            "role",
        ),
        extra=1,
        can_delete=True,
    )
    author = project.author
    list_delete_user = []

    delete_error = False
    author_exist = False
    author_admin_error = False
    if request.method == "POST":
        formset = ManageProjectUserFormset(request.POST, instance=project)
        if formset.is_valid():
            for form in formset:
                cd = form.cleaned_data
                if cd.get("user") == author:
                    author_exist = True
                    if cd.get("role") != "administrator":
                        author_admin_error = True
                    if cd.get("DELETE") is True:
                        delete_error = True
            if (
                author_exist is False
                or delete_error is True
                or author_admin_error is True
            ):
                formset = ManageProjectUserFormset(instance=project)
                return render(
                    request,
                    "mapping/mapping_project_user.html",
                    {
                        "form": formset,
                        "current_proj": current_proj,
                        "data_type": data_type,
                        "current_user": current_user,
                        "data_type": data_type,
                        "mpu_list": mpu_list,
                        "project": project,
                    },
                )
            for form in formset.deleted_forms:
                cd_delete = form.cleaned_data
                user_delete = cd_delete.get("user")
                if user_delete is not None:
                    if user_delete != author:
                        list_delete_user.append(form.cleaned_data)
            if list_delete_user != []:
                obj_to_delete = []
                users_collection = []
                obj_str = ""
                for user in list_delete_user:
                    obj_str = str(obj_str) + str(user.get("id").pk) + ","
                    obj_to_delete.append(user.get("id"))
                    users_collection.append(user.get("user"))
                form = LinkMappingUserForm

                return render(
                    request,
                    "mapping/form_delete_users.html",
                    {
                        "form": form,
                        "obj_to_delete": obj_to_delete,
                        "users": users_collection,
                        "data_type": "user_mapping",
                        "project_id": current_proj,
                        "project": project,
                        "obj_str": obj_str,
                    },
                )
            else:
                for form in formset:
                    cd = form.cleaned_data
                    logger.debug(cd)
                    user = cd.get("user")
                    role = cd.get("role")
                    if user is not None:
                        if user != author:
                            if role == "administrator":
                                assign_perm("view_project", user.user, project)
                                assign_perm("add_project", user.user, project)
                                assign_perm("change_project", user.user, project)
                                assign_perm("delete_project", user.user, project)
                                assign_perm("manage_user", user.user, project)
                                messages.info(
                                    request,
                                    "User "
                                    + str(user.user)
                                    + " set as administrator for project "
                                    + str(project),
                                )
                            elif role == "contributor":
                                assign_perm("view_project", user.user, project)
                                assign_perm("change_project", user.user, project)
                                assign_perm("delete_project", user.user, project)
                                remove_perm("add_project", user.user, project)
                                remove_perm("manage_user", user.user, project)
                                messages.info(
                                    request,
                                    "User "
                                    + str(user.user)
                                    + " set as contributor for project "
                                    + str(project),
                                )
                            elif role == "collaborator":
                                assign_perm("view_project", user.user, project)
                                remove_perm("add_project", user.user, project)
                                remove_perm("change_project", user.user, project)
                                remove_perm("delete_project", user.user, project)
                                messages.info(
                                    request,
                                    "User "
                                    + str(user.user)
                                    + " set as collaborator for project "
                                    + str(project),
                                )
                formset.save()
                return HttpResponseRedirect(
                    reverse("core:mapping_project_user", args=[current_proj])
                )

    else:
        formset = ManageProjectUserFormset(instance=project)

    return render(
        request,
        "mapping/mapping_project_user.html",
        {
            "form": formset,
            "current_proj": current_proj,
            "data_type": data_type,
            "current_user": current_user,
            "data_type": data_type,
            "mpu_list": mpu_list,
            "project": project,
        },
    )


def get_list_mapping_type_of_available_children(list_ressource, data_type, connector):
    i = 0
    list_objects = []
    list_objects_map = []
    for item in list_ressource:
        if item == data_type:
            data_type_map = list_ressource[item]
            i = 1
        else:
            if i == 1:
                list_objects.append(items.get_plural_form_of_type(item))
                for res in list_ressource[item]:
                    list_objects_map.append(connector.get_plural_form_of_type(res))
    return data_type_map, list_objects, list_objects_map



from django import template
from openlink.core.connector import Publisher
from openlink.core.models import Tool

# from openlink.core.lib.tools import
register = template.Library()


@register.filter(name="is_publisher")
def is_publisher(instance):
    return issubclass(
        Tool.objects.get(id=instance.id).get_connector().__class__, Publisher
    )



from django.test import TestCase  # noqa
from django.utils import timezone

from .models import Profile, Tool

# Create your tests here.


class toolTest(TestCase):
    def test_create_tool(self):
        author = Profile.objects.get(id=1)

        seaf = Tool.objects.create(
            name="name",
            connector="SeafileConnector",
            author=author,
            date_created=timezone.now(),
        )
        self.assertIs(seaf.get_connector(), "seafile")



import logging

from crispy_forms.helper import FormHelper
from crispy_forms.layout import HTML, Button, Field, Fieldset, Layout, Submit
from django import forms
from openlink.core import connector as tools

from .models import (
    Assay,
    Dataset,
    Investigation,
    Mapping,
    MappingProjectUser,
    Project,
    Study,
    Tool,
)

# from django.forms import fields

logger = logging.getLogger(__name__)


class BaseForm(forms.Form):
    parent_id = int

    def __init__(self, *args, **kwargs):
        super(BaseForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_group_wrapper_class = "row"
        self.helper.label_class = "col-sm-offset-1 col-sm-2"
        self.helper.field_class = "col-md-8"
        self.helper.add_input(
            Button(
                "back",
                "Back",
                css_class="btn-secondary",
                onClick="javascript:history.go(-1);",
            )
        )

    def clean(self):
        clean_data = super().clean()
        name = clean_data["name"]
        type_class = (
            str(self.__class__)
            .replace("<class 'openlink.core.forms.", "")
            .replace("Form'>", "")
        )
        if not type_class == "Project":
            parent_type = eval(type_class).get_parent_type()
            if not type(self.parent_id) == type:
                parent = getattr(parent_type, "get_" + parent_type.__name__.lower())(
                    self.parent_id
                )
                same_level = parent.first().items
                if any(d.name == name for d in same_level):
                    self.add_error(
                        "name",
                        "a " + type_class + " with the same name already exist",
                    )


class SelectObjectTool(forms.Form):
    object_name = forms.ChoiceField(choices=(), label="unlinked_objects")

    def __init__(self, unlinked_objects=None, *args, **kwargs):
        super(SelectObjectTool, self).__init__(*args, **kwargs)
        self.fields["object_name"].widget = forms.RadioSelect()

        if unlinked_objects:
            self.fields["object_name"].choices = unlinked_objects


class SelectMultipleObjectOption(forms.Form):
    def __init__(self, *args, **kwargs):
        list_objects = kwargs.pop("list_objects")
        list_objects_map = kwargs.pop("list_objects_map")
        data_type_map = kwargs.pop("data_type_map")[0]
        has_mapping_options = kwargs.pop("has_mapping_options")
        connector = kwargs.pop("tool_connector")
        action = kwargs.pop("action")
        super(SelectMultipleObjectOption, self).__init__(*args, **kwargs)
        if len(list_objects_map) > 0:
            self.fields["create_and_link_children"] = forms.BooleanField(
                widget=forms.CheckboxInput(),
                required=False,
                label='Create and link any %s available in "%s"'
                % (" ".join(str(x) for x in list_objects_map), data_type_map),
                help_text="Get links from the content of the %s and create %s in Openlink"
                % (data_type_map, " ".join(str(x) for x in list_objects)),
            )
        if has_mapping_options:
            list_extra_options = connector.get_fields_option_mapping(self.fields)
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.form_action = action
        self.helper.layout = Layout(
            HTML(
                """
                <h4 class="display-titre" style="background-color: rgb(255, 255, 255)">Choose options for selected {% for value in data_type_map %}{{value}}{% if not forloop.last %} or{% endif %}{% endfor %}
                {{object_name}}</h4>
                <br>
                """
            ),
            Field("create_and_link_children"),
        )
        if has_mapping_options:
            list_extra_options = connector.get_fields_option_mapping(self.fields)
            self.helper.layout.append(
                Fieldset("{{tool_name}} options", *list_extra_options),
            )
        else:
            if len(self.fields) == 0:
                self.helper.layout.append(
                    Layout(
                        HTML(
                            """
                        {% if not has_mapping_options %}
                            <h6>There are no options to display, please proceed.</h6>
                        {% endif %}
                        """
                        ),
                    )
                )


class SelectObjectOption(forms.Form):

    replace_openlink_name = forms.BooleanField(
        widget=forms.CheckboxInput(), required=False
    )
    use_description = forms.BooleanField(
        label="Use this description", widget=forms.CheckboxInput(), required=False
    )
    add_description = forms.BooleanField(
        label="Add this description to current description",
        widget=forms.CheckboxInput(),
        required=False,
    )

    def __init__(self, *args, **kwargs):
        data_type = kwargs.pop("data_type")
        list_objects = kwargs.pop("list_objects")
        list_objects_map = kwargs.pop("list_objects_map")
        data_type_map = kwargs.pop("data_type_map")[0]
        object_name = kwargs.pop("object_name")
        has_mapping_options = kwargs.pop("has_mapping_options")
        object_desc = kwargs.pop("object_desc")
        connector = kwargs.pop("tool_connector")
        action = kwargs.pop("action")
        super(SelectObjectOption, self).__init__(*args, **kwargs)
        self.fields[
            "replace_openlink_name"
        ].label = 'Replace openlink %s name by "%s"' % (data_type, object_name)
        self.fields[
            "replace_openlink_name"
        ].help_text = 'Update the %s name with the name of the %s "%s"' % (
            data_type,
            data_type_map,
            object_name,
        )
        self.fields["use_description"].help_text = (
            "Use the description found by openlink as description for the current %s"
            % (data_type)
        )
        self.fields["add_description"].help_text = (
            "Add the description found by openlink to the already existing %s description"
            % (data_type)
        )
        if len(list_objects_map) > 0:
            self.fields["create_and_link_children"] = forms.BooleanField(
                widget=forms.CheckboxInput(),
                required=False,
                label='Create and link any %s available in "%s"'
                % (" ".join(str(x) for x in list_objects_map), object_name),
                help_text="Get links from the content of the %s and create %s in Openlink"
                % (data_type_map, " ".join(str(x) for x in list_objects)),
            )

        if not object_desc:
            self.fields["use_description"].widget.attrs["disabled"] = True
            self.fields["add_description"].widget.attrs["disabled"] = True
        if has_mapping_options:
            list_extra_options = connector.get_fields_option_mapping(self.fields)
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.form_action = action
        self.helper.layout = Layout(
            HTML(
                """
                <h4 class="display-titre" style="background-color: rgb(255, 255, 255)">Select options for the selected {% for value in data_type_map %}{{value}}{% if not forloop.last %} or{% endif %}{% endfor %}
                {{object_name}}:</h4>
                """
            ),
            Field("replace_openlink_name"),
            Field("create_and_link_children"),
            HTML(
                """
                <h4 class="display-titre" style="background-color: rgb(255, 255, 255)">Description options </h4>
                """
            ),
            Field("use_description"),
            Field("add_description"),
            HTML(
                """
                {% if object_desc %}
                Description found for this {% for value in data_type_map %}{{value}}{% if not forloop.last %} or{% endif %}{% endfor %} {{object_name}}:
                <div class="description_found">
                    {{ object_desc | safe }}
                </div>
                {% endif %}
                <br>"""
            ),
        )
        if has_mapping_options:
            list_extra_options = connector.get_fields_option_mapping(self.fields)
            self.helper.layout.append(
                Fieldset("{{tool_name}} options", *list_extra_options),
            )


class ProjectForm(BaseForm, forms.ModelForm):
    class Meta:
        model = Project
        fields = ["name", "description"]

        widgets = {
            "description": forms.Textarea(attrs={"rows": 3, "cols": 20}),
        }

    def __init__(self, *args, **kwargs):
        super(ProjectForm, self).__init__(*args, **kwargs)
        self.helper.add_input(Submit("submit", "Add project", css_class="btn-primary"))


class InvestigationForm(BaseForm, forms.ModelForm):
    class Meta:
        model = Investigation
        fields = ["name", "description"]

        widgets = {
            "description": forms.Textarea(attrs={"rows": 3, "cols": 20}),
        }

    def __init__(self, *args, **kwargs):
        super(InvestigationForm, self).__init__(*args, **kwargs)
        self.helper.add_input(
            Submit("submit", "Add investigation", css_class="btn-primary")
        )


class StudyForm(BaseForm, forms.ModelForm):
    class Meta:
        model = Study
        fields = ["name", "description"]

    def __init__(self, *args, **kwargs):
        super(StudyForm, self).__init__(*args, **kwargs)
        self.helper.add_input(Submit("submit", "Add Study", css_class="btn-primary"))


class SelectStudyForm(forms.Form):
    studies_name = forms.ModelChoiceField(queryset=Study.objects.all())

    def __init__(self, *args, **kwargs):
        investigation = kwargs.pop("instance")
        super(SelectStudyForm, self).__init__(*args, **kwargs)
        self.fields["studies_name"].widget = forms.Select()

        if investigation:
            self.fields["studies_name"].queryset = investigation.studies.all()


class SelectAssayForm(forms.Form):
    assays_name = forms.ModelChoiceField(queryset=Assay.objects.all())

    def __init__(self, *args, **kwargs):
        investigation = kwargs.pop("instance")
        super(SelectAssayForm, self).__init__(*args, **kwargs)
        self.fields["assays_name"].widget = forms.Select()

        if investigation:
            investigation_id = investigation.id
            self.fields["assays_name"].queryset = Assay.objects.filter(
                study__investigation__id=investigation_id
            )


class AssayForm(BaseForm, forms.ModelForm):
    class Meta:
        model = Assay
        fields = ("name", "description")
        widgets = {
            "description": forms.Textarea(attrs={"rows": 3, "cols": 20}),
        }

    def __init__(self, *args, **kwargs):
        super(AssayForm, self).__init__(*args, **kwargs)
        self.helper.add_input(Submit("submit", "Add assay", css_class="btn-primary"))


class DatasetForm(BaseForm, forms.ModelForm):
    class Meta:
        model = Dataset
        fields = ["name", "description"]
        widgets = {
            "description": forms.Textarea(attrs={"rows": 3, "cols": 20}),
        }

    def __init__(self, *args, **kwargs):
        super(DatasetForm, self).__init__(*args, **kwargs)
        self.helper.add_input(Submit("submit", "Add dataset", css_class="btn-primary"))


class ToolForm(forms.Form):

    connectors = [
        (connector.__name__, connector.get_name())
        for connector in tools.get_connectors()
    ]

    connector = forms.ChoiceField(choices=connectors, required=True)
    name = forms.CharField(
        initial=connectors[0][1], label="Name", max_length=100, required=True
    )

    def clean(self):
        cleaned_data = super().clean()
        logger.debug(cleaned_data)
        if Tool.objects.filter(
            name=cleaned_data["name"], project_id=self.project_id
        ).first():
            self.add_error("name", "a tool with the same name already exists ")
        return cleaned_data


class ToolProjectForm(forms.ModelForm, BaseForm):
    class Meta:
        model = Tool
        fields = [
            "name",
        ]

    def __init__(self, *args, **kwargs):
        super(ToolProjectForm, self).__init__(*args, **kwargs)
        self.helper.add_input(Submit("submit", "Add tool", css_class="btn-primary"))


class SelectToolForm(forms.Form):
    Name = forms.ChoiceField(choices=[], required=True)

    def __init__(self, *args, **kwargs):
        tool_list = kwargs.pop("instance")
        super(SelectToolForm, self).__init__(*args, **kwargs)
        self.fields["Name"].widget = forms.RadioSelect()
        tools_choices = []
        for tool in tool_list:
            tools_choices.append((tool.id, tool.get_param("url")))

        self.fields["Name"].choices = tools_choices


class SelectProjectTool(forms.Form):

    project_name = forms.ChoiceField(choices=(), label="Projects")
    replace_openlink_object_name = forms.BooleanField(
        widget=forms.CheckboxInput(), required=False
    )

    def __init__(self, projects=None, *args, **kwargs):
        super(SelectProjectTool, self).__init__(*args, **kwargs)
        self.fields["project_name"].widget = forms.RadioSelect()
        if projects:
            self.fields["project_name"].choices = projects


class SelectInvestigationTool(forms.Form):

    investigation_name = forms.ChoiceField(choices=(), label="Investigations")
    replace_openlink_investigation_name = forms.BooleanField(
        widget=forms.CheckboxInput(), required=False
    )
    create_associated_objects_in_openlink = forms.BooleanField(
        widget=forms.CheckboxInput(), required=False
    )

    def __init__(self, investigations=None, *args, **kwargs):
        super(SelectInvestigationTool, self).__init__(*args, **kwargs)
        self.fields["investigation_name"].widget = forms.RadioSelect()
        if investigations:
            self.fields["investigation_name"].choices = investigations


class SelectInvestigationToolMapping(forms.Form):

    investigation_name = forms.ChoiceField(choices=(), label="Investigations")

    def __init__(self, investigations=None, *args, **kwargs):
        super(SelectInvestigationToolMapping, self).__init__(*args, **kwargs)
        self.fields["investigation_name"].widget = forms.RadioSelect()
        if investigations:
            self.fields["investigation_name"].choices = investigations


class SelectProjectToolMapping(forms.Form):

    project_name = forms.ChoiceField(choices=(), label="Projects")

    def __init__(self, projects=None, *args, **kwargs):
        super(SelectProjectToolMapping, self).__init__(*args, **kwargs)
        self.fields["project_name"].widget = forms.RadioSelect()
        if projects:
            self.fields["project_name"].choices = projects


class SelectStudyTool(forms.Form):

    study_name = forms.ChoiceField(choices=(), label="Studies")
    replace_openlink_study_name = forms.BooleanField(
        widget=forms.CheckboxInput(), required=False
    )

    def __init__(self, studies=None, *args, **kwargs):
        super(SelectStudyTool, self).__init__(*args, **kwargs)
        self.fields["study_name"].widget = forms.RadioSelect()
        if studies:
            self.fields["study_name"].choices = studies


class SelectStudiesTool(forms.Form):

    study_name = forms.ChoiceField(choices=(), label="Studies")

    def __init__(self, studies=None, *args, **kwargs):
        super(SelectStudiesTool, self).__init__(*args, **kwargs)
        self.fields["study_name"].widget = forms.CheckboxSelectMultiple()
        if studies:
            self.fields["study_name"].choices = studies


class SelectAssayTool(forms.Form):

    assay_name = forms.ChoiceField(choices=(), label="Assays")
    replace_openlink_assay_name = forms.BooleanField(
        widget=forms.CheckboxInput(), required=False
    )
    create_associated_objects_in_openlink = forms.BooleanField(
        widget=forms.CheckboxInput(), required=False
    )

    def __init__(self, assays=None, *args, **kwargs):
        super(SelectAssayTool, self).__init__(*args, **kwargs)
        self.fields["assay_name"].widget = forms.RadioSelect()
        if assays:
            self.fields["assay_name"].choices = assays


class SelectAssaysTool(forms.Form):

    assay_name = forms.ChoiceField(choices=(), label="Assay")

    def __init__(self, assays=None, *args, **kwargs):
        super(SelectAssaysTool, self).__init__(*args, **kwargs)
        self.fields["assay_name"].widget = forms.CheckboxSelectMultiple()

        if assays:
            self.fields["assay_name"].choices = assays


class SelectDatasetTool(forms.Form):

    dataset_name = forms.ChoiceField(choices=(), label="Datasets")
    replace_openlink_dataset_name = forms.BooleanField(
        widget=forms.CheckboxInput(), required=False
    )
    synchronize_dataset_with_experiment = forms.BooleanField(
        widget=forms.CheckboxInput(), required=False
    )

    def __init__(self, datasets=None, *args, **kwargs):
        super(SelectDatasetTool, self).__init__(*args, **kwargs)
        self.fields["dataset_name"].widget = forms.RadioSelect()

        if datasets:
            self.fields["dataset_name"].choices = datasets


class SelectDatasetsTool(forms.Form):

    dataset_name = forms.ChoiceField(choices=(), label="Datasets")

    def __init__(self, datasets=None, *args, **kwargs):
        super(SelectDatasetsTool, self).__init__(*args, **kwargs)
        self.fields["dataset_name"].widget = forms.CheckboxSelectMultiple()
        if datasets:
            self.fields["dataset_name"].choices = datasets


class LinkProjectForm(forms.ModelForm):
    class Meta:
        model = Project
        fields = [
            "name",
        ]


class LinkInvestigationForm(forms.ModelForm):
    class Meta:
        model = Investigation
        fields = [
            "name",
        ]


class LinkDatasetForm(forms.ModelForm):
    class Meta:
        model = Dataset
        fields = [
            "name",
        ]


class LinkAssayForm(forms.ModelForm):
    class Meta:
        model = Assay
        fields = [
            "name",
        ]


class LinkStudyForm(forms.ModelForm):
    class Meta:
        model = Study
        fields = [
            "name",
        ]


class LinkToolForm(forms.ModelForm):
    class Meta:
        model = Tool
        fields = [
            "name",
        ]


class LinkMappingForm(forms.ModelForm):
    class Meta:
        model = Mapping
        fields = [
            "name",
        ]


class LinkMappingUserForm(forms.ModelForm):
    class Meta:
        model = MappingProjectUser
        fields = [
            "role",
        ]


class SelectAssaysForm(forms.Form):
    assays_name = forms.ModelChoiceField(queryset=Assay.objects.all())

    def __init__(self, *args, **kwargs):
        investigation = kwargs.pop("investigation")
        super(SelectAssaysForm, self).__init__(*args, **kwargs)
        self.fields["assays_name"].widget = forms.RadioSelect()

        if investigation:
            self.fields["assays_name"].queryset = Assay.objects.filter(
                study__investigation__id=investigation.id
            )


class SelectStudiesForm(forms.Form):
    studies_name = forms.ModelChoiceField(queryset=Study.objects.all())

    def __init__(self, *args, **kwargs):
        investigation = kwargs.pop("investigation")
        super(SelectStudiesForm, self).__init__(*args, **kwargs)
        self.fields["studies_name"].widget = forms.RadioSelect()

        if investigation:
            self.fields["studies_name"].queryset = Study.objects.filter(
                investigation__id=investigation.id
            )


ADMINISTRATOR = "administrator"
CONTRIBUTOR = "contributor"
COLLABORATER = "collaborater"
STATUS = [
    (ADMINISTRATOR, "administrator"),
    (CONTRIBUTOR, "contributor"),
    (COLLABORATER, "collaborater"),
]


class ManageProjectUserForm(forms.Form):
    role = forms.ChoiceField(choices=STATUS)

    def __init__(self, *args, **kwargs):
        project = kwargs.pop("project")
        super(ManageProjectUserForm, self).__init__(*args, **kwargs)

        if project:
            self.initial["role"] = MappingProjectUser.objects.get(project=project)


class AddProjectUserForm(forms.ModelForm):
    class Meta:
        model = MappingProjectUser
        fields = ("user", "role")

    def __init__(self, *args, **kwargs):
        super(AddProjectUserForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_group_wrapper_class = "row"
        self.helper.label_class = "col-sm-offset-1 col-sm-2"
        self.helper.field_class = "col-md-8"
        self.helper.add_input(Submit("submit", "Add", css_class="btn-primary"))



import logging

from django.apps import apps
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseForbidden, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.urls import reverse
from django.utils import timezone
from django.views.decorators.http import require_GET, require_POST
from guardian.decorators import permission_required_or_403
from guardian.shortcuts import assign_perm
from openlink.core.connector import Publisher
from openlink.core.filters import ProjectFilter
from openlink.core.forms import (
    LinkAssayForm,
    LinkDatasetForm,
    LinkInvestigationForm,
    LinkMappingForm,
    LinkProjectForm,
    LinkStudyForm,
    ProjectForm,
)
from openlink.core.models import Mapping, MappingProjectUser, Profile, Project, Tool
from openlink.core.views import users

logger = logging.getLogger(__name__)


@login_required
def projects(request):
    # Display all projects shared or owned by the user
    list_ids = []
    projects_list_map2 = MappingProjectUser.objects.filter(user__user=request.user)
    for proj_map in projects_list_map2:
        list_ids.append(proj_map.project.id)
    projects_list = Project.objects.filter(id__in=list_ids)
    projects_filter = ProjectFilter(request.GET, queryset=projects_list)
    return render(request, "projects/list.html", {"filter": projects_filter})


@login_required
@require_GET
def new_project(request):
    # Display an empty form to add a new project
    return render(request, "new_item/new_item.html", {"form": ProjectForm()})


@login_required
@require_POST
def add_project(request):
    # Save the project with data from the project form
    form = ProjectForm(request.POST)

    if not form.is_valid():
        # Display the same empty project form if form is not valid
        return render(request, "new_item/new_item.html", {"form": form})

    project = form.save(commit=False)
    profile = get_object_or_404(Profile, user=request.user)
    project.author = profile
    project.date_created = timezone.now()
    project.save()
    assign_perm("view_project", request.user, project)
    assign_perm("add_project", request.user, project)
    assign_perm("change_project", request.user, project)
    assign_perm("delete_project", request.user, project)
    assign_perm("manage_user", request.user, project)

    new_mappingprojectuser = MappingProjectUser()
    new_mappingprojectuser.user = get_object_or_404(Profile, user=request.user)
    new_mappingprojectuser.project = project
    new_mappingprojectuser.role = "administrator"
    new_mappingprojectuser.save()
    messages.info(request, "Project " + str(project) + " created")

    return HttpResponseRedirect(reverse("core:projects"))


def convert_to_byte(value):
    # convert octet to byte
    value_split = value.split(" ")
    try:
        nb = float(value_split[0])
    except ValueError:
        return float(0)
    byte_format = str(value_split[1])
    if byte_format == "B":
        byte_value = float(nb)
    elif byte_format == "KB":
        byte_value = float(nb * 1024)
    elif byte_format == "MB":
        byte_value = float(nb * 1024**2)
    elif byte_format == "GB":
        byte_value = float(nb * (1024**3))
    elif byte_format == "TB":
        byte_value = float(nb * (1024**4))
    elif byte_format == "PB":
        byte_value = float(nb * (1024**5))
    else:
        byte_value = float(0)
    return byte_value


def humansize(nbytes):
    # make the filesize human readable
    suffixes = ["B", "KB", "MB", "GB", "TB", "PB"]
    i = 0
    while nbytes >= 1024 and i < len(suffixes) - 1:
        nbytes /= 1024.0
        i += 1
    f = ("%.2f" % nbytes).rstrip("0").rstrip(".")
    return "%s %s" % (f, suffixes[i])


@login_required
@permission_required_or_403("view_project", (Project, "id", "project_id"))
def details(request, project_id):
    logger.debug("details")
    # Display the complete project page from his ID
    project = get_object_or_404(Project, pk=project_id)
    access = users.get_access_level(request, project_id)
    progress_bar, space_info_list_2 = progressbar(project, project)
    return render(
        request,
        "projects/details.html",
        {
            "project": project,
            "access": access,
            "space_info_list": space_info_list_2,
            "progress_bar": progress_bar,
        },
    )


@login_required
@permission_required_or_403("delete_project", (Project, "id", "project_id"))
def delete(request, *args, **kwargs):
    # Delete an openlink object
    project_id = kwargs["project_id"]
    project = get_object_or_404(Project, id=project_id)
    obj_id = kwargs["pk"]
    data_type = kwargs["data_type"]
    Model = apps.get_model("core", data_type.capitalize())
    obj_to_delete = get_object_or_404(Model, id=obj_id)
    if request.method == "POST":
        if data_type == "mapping":
            obj_to_delete = get_object_or_404(Mapping, id=obj_id)
            obj_to_delete.delete()

            messages.info(request, str(obj_to_delete) + " deleted")

            return HttpResponseRedirect(
                reverse("core:projects-details", kwargs={"project_id": project_id})
            )
        else:
            if data_type == "project":
                if not request.user.has_perm("manage_user", project):
                    return HttpResponseForbidden()
                else:
                    obj_to_delete.delete()
                    messages.info(request, str(obj_to_delete) + " deleted")
                    return HttpResponseRedirect(reverse("core:projects"))
            else:
                obj_to_delete.delete()
                messages.info(request, str(obj_to_delete) + " deleted")

        return HttpResponseRedirect(
            reverse("core:projects-details", kwargs={"project_id": project_id})
        )

    else:
        if data_type == "project":
            if not request.user.has_perm("manage_user", project):
                return HttpResponseForbidden()
            else:
                form = LinkProjectForm
        if data_type == "investigation":
            form = LinkInvestigationForm
        if data_type == "dataset":
            form = LinkDatasetForm
        if data_type == "assay":
            form = LinkAssayForm
        if data_type == "study":
            form = LinkStudyForm
        if data_type == "mapping":
            form = LinkMappingForm
    return render(
        request,
        "mapping/form_delete.html",
        {
            "form": form,
            "obj": obj_to_delete,
            "data_type": kwargs["data_type"],
            "project_id": project_id,
        },
    )


def progressbar(item, project):
    # Build progressbar of filesize
    space_info_list = []
    all_tools = []
    progress_bar = 0
    all_tools = Tool.objects.filter(project=project)
    obj = apps.get_model("core", str(item.type))
    all_datasets = obj.get_dataset(item.id)
    for tool in all_tools:
        if not issubclass(tool.get_connector(), Publisher):
            dataset_exists = False
            sum_space = 0
            for dataset in all_datasets:
                if Mapping.objects.filter(
                    foreign_id_obj__dataset=dataset, tool_id=tool
                ).exists():
                    dataset_exists = True
                    maps = Mapping.objects.filter(
                        foreign_id_obj__dataset=dataset, tool_id=tool
                    )
                    for mapping in maps:
                        size = mapping.size
                        if size is not None:
                            sum_space += size
            if dataset_exists is True:
                space_info_list.append(
                    {"size_total": sum_space, "tool_name": tool.name, "tool": tool}
                )
    sum_space = 0
    list_tool = []
    space_info_list_2 = []
    tool_name = ""
    tool = ""
    total_sum_space = 0
    for space in space_info_list:
        if space["tool_name"] not in list_tool:
            sum_space = space["size_total"]
            total_sum_space += sum_space
            tool_name = space["tool_name"]
            tool = space["tool"]
            space_info_list_2.append(
                {
                    "size_total": humansize(sum_space),
                    "tool_name": tool_name,
                    "tool": tool,
                }
            )
            list_tool.append(space["tool_name"])
        else:
            sum_space = sum_space + space["size_total"]
            for d in space_info_list_2:
                d.update(
                    ("size_total", humansize(sum_space))
                    for k, v in d.items()
                    if d["tool_name"] == tool_name
                )
    for value in space_info_list_2:
        if value["size_total"] != "0 B":
            percent_space = convert_to_byte(value["size_total"]) / total_sum_space * 100
            value["percent_space"] = percent_space
            progress_bar = 1
        else:
            value["percent_space"] = 0
    space_info_list_2.append(
        {"size_total": humansize(total_sum_space), "tool_name": "total", "tool": -1}
    )
    return progress_bar, space_info_list_2



from django import template
from openlink.core.models import Assay

register = template.Library()


@register.filter(name="contains_assay")
def contains_assay(obj):
    contains = False
    list_assays = Assay.objects.filter(study__investigation__id=obj.id)
    if list_assays:
        contains = True
    return contains



import logging

import django.dispatch
from django.apps import apps
from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver
from openlink.core.models import Mapping, Tool

logger = logging.getLogger(__name__)

signal = django.dispatch.Signal()
new_mapping_signal = django.dispatch.Signal()
delete_signal = django.dispatch.Signal()


@receiver(pre_delete, sender=Mapping)
def delete__signal(sender, **kwargs):
    mapping = kwargs["instance"]
    item = apps.get_model("core", str(mapping.type)).objects.get(
        id=mapping.foreign_id_obj.id
    )
    connector = Tool.objects.get(mapping__id=mapping.id).get_connector()
    delete_signal.send_robust(
        apps.get_model("core", str(mapping.type)),
        del_mapping=mapping,
        item=item,
        connector=connector,
    )


@receiver(post_save, sender=Mapping)
def new_map_signal(sender, **kwargs):
    logger.debug("new map")
    mapping = kwargs["instance"]
    item = apps.get_model("core", str(mapping.type)).objects.get(
        id=mapping.foreign_id_obj.id
    )
    connector = Tool.objects.get(mapping__id=mapping.id).get_connector()
    new_mapping_signal.send_robust(
        apps.get_model("core", str(mapping.type)),
        new_mapping=mapping,
        item=item,
        connector=connector,
    )
